@@ -24,7 +24,7 @@ using namespace std;
 namespace {
 	// For tracking the player's average income, store daily net worth over this
 	// number of days.
-	static const unsigned HISTORY = 100;
+	const unsigned HISTORY = 100;
 }
 
 
@@ -55,10 +55,7 @@ void Account::Load(const DataNode &node)
 		else if(child.Token(0) == "score" && child.Size() >= 2)
 			creditScore = child.Value(1);
 		else if(child.Token(0) == "mortgage")
-		{
-			mortgages.push_back(Mortgage(0, 0, 0));
-			mortgages.back().Load(child);
-		}
+			mortgages.emplace_back(child);
 		else if(child.Token(0) == "history")
 			for(const DataNode &grand : child)
 				history.push_back(grand.Value(0));
@@ -136,7 +133,7 @@ string Account::Step(int64_t assets, int64_t salaries)
 	
 	// Keep track of what payments were made and whether any could not be made.
 	salariesOwed += salaries;
-	bool paid = true;
+	bool missedPayment = false;
 	
 	// Crew salaries take highest priority.
 	int64_t salariesPaid = salariesOwed;
@@ -149,8 +146,8 @@ string Account::Step(int64_t assets, int64_t salaries)
 			salariesPaid = max<int64_t>(credits, 0);
 			salariesOwed -= salariesPaid;
 			credits -= salariesPaid;
-			paid = false;
-			out << "You could not pay all your crew salaries. ";
+			missedPayment = true;
+			out << "You could not pay all your crew salaries.";
 		}
 		else
 		{
@@ -169,9 +166,9 @@ string Account::Step(int64_t assets, int64_t salaries)
 		if(payment > credits)
 		{
 			mortgage.MissPayment();
-			if(paid)
-				out << "You missed a mortgage payment. ";
-			paid = false;
+			if(!missedPayment)
+				out << "You missed a mortgage payment.";
+			missedPayment = true;
 		}
 		else
 		{
@@ -202,30 +199,37 @@ string Account::Step(int64_t assets, int64_t salaries)
 	// If you failed to pay any debt, your credit score drops. Otherwise, even
 	// if you have no debts, it increases. (Because, having no debts at all
 	// makes you at least as credit-worthy as someone who pays debts on time.)
-	creditScore = max(200, min(800, creditScore + (paid ? 1 : -5)));
+	creditScore = max(200, min(800, creditScore + (missedPayment ? -5 : 1)));
 	
 	// If you didn't make any payments, no need to continue further.
 	if(!(salariesPaid + mortgagesPaid + finesPaid))
 		return out.str();
+	else if(missedPayment)
+		out << " ";
 	
 	out << "You paid ";
 	
+	auto creditString = [](int64_t payment) -> string
+	{
+		return payment == 1 ? "1 credit" : Format::Credits(payment) + " credits";
+	};
+	
 	// If you made payments of all three types, the punctuation needs to
 	// include commas, so just handle that separately here.
 	if(salariesPaid && mortgagesPaid && finesPaid)
-		out << Format::Number(salariesPaid) << " credits in crew salaries, " << Format::Number(mortgagesPaid)
-			<< " in mortgages, and " << Format::Number(finesPaid) << " in fines.";
+		out << creditString(salariesPaid) << " in crew salaries, "
+			<< creditString(mortgagesPaid) << " in mortgages, and "
+			<< creditString(finesPaid) << " in fines.";
 	else
 	{
 		if(salariesPaid)
-			out << Format::Number(salariesPaid) << ((mortgagesPaid || finesPaid) ?
-				" credits in crew salaries and " : " credits in crew salaries.");
+			out << creditString(salariesPaid) << " in crew salaries"
+				<< ((mortgagesPaid || finesPaid) ? " and " : ".");
 		if(mortgagesPaid)
-			out << Format::Number(mortgagesPaid) << (salariesPaid ? " " : " credits ")
-				<< (finesPaid ? "in mortgage payments and " : "in mortgage payments.");
+			out << creditString(mortgagesPaid) << " in mortgages"
+				<< (finesPaid ? " and " : ".");
 		if(finesPaid)
-			out << Format::Number(finesPaid) << ((salariesPaid || mortgagesPaid) ?
-				" in fines." : " credits in fines.");
+			out << creditString(finesPaid) << " in fines.";
 	}
 	return out.str();
 }
@@ -311,6 +315,19 @@ int Account::CreditScore() const
 
 
 
+// Get the total amount owed for "Mortgage", "Fine", or both.
+int64_t Account::TotalDebt(const string &type) const
+{
+	int64_t total = 0;
+	for(const Mortgage &mortgage : mortgages)
+		if(type.empty() || mortgage.Type() == type)
+			total += mortgage.Principal();
+	
+	return total;
+}
+
+
+
 // Extrapolate from the player's current net worth history to determine how much
 // their net worth is expected to change over the course of the next year.
 int64_t Account::YearlyRevenue() const
