@@ -65,7 +65,7 @@ const Sprite *Body::GetSprite() const
 // Get the width of this object, in world coordinates (i.e. taking zoom into account).
 double Body::Width() const
 {
-	return sprite ? (.5 * zoom) * sprite->Width() : 0.;
+	return static_cast<double>(sprite ? (.5f * zoom) * sprite->Width() : 0.f);
 }
 
 
@@ -73,7 +73,7 @@ double Body::Width() const
 // Get the height of this object, in world coordinates (i.e. taking zoom into account).
 double Body::Height() const
 {
-	return sprite ? (.5 * zoom) * sprite->Height() : 0.;
+	return static_cast<double>(sprite ? (.5f * zoom) * sprite->Height() : 0.f);
 }
 
 
@@ -94,39 +94,27 @@ int Body::GetSwizzle() const
 
 
 
-// Get the sprite for the given time step. If no time step is given, this will
-// return the frame from the most recently given step.
-Body::Frame Body::GetFrame(int step) const
+// Get the frame index for the given time step. If no time step is given, this
+// will return the frame from the most recently given step.
+float Body::GetFrame(int step) const
 {
-	SetStep(step, Screen::IsHighResolution());
+	if(step >= 0)
+		SetStep(step);
+	
 	return frame;
 }
 
 
 
-Body::Frame Body::GetFrame(int step, bool isHighDPI) const
-{
-	SetStep(step, isHighDPI);
-	return frame;
-}
-
-
-
-int Body::GetFrameIndex(int step) const
-{
-	SetStep(step, Screen::IsHighResolution());
-	return activeIndex;
-}
-
-
-
 // Get the mask for the given time step. If no time step is given, this will
 // return the mask from the most recently given step.
 const Mask &Body::GetMask(int step) const
 {
-	SetStep(step, currentHighDPI);
+	if(step >= 0)
+		SetStep(step);
+	
 	static const Mask EMPTY;
-	return sprite ? sprite->GetMask(activeIndex) : EMPTY;
+	return sprite ? sprite->GetMask(round(frame)) : EMPTY;
 }
 
 
@@ -164,10 +152,18 @@ Point Body::Unit() const
 
 
 
-// Zoom factor. THis controls how big the sprite should be drawn.
+// Zoom factor. This controls how big the sprite should be drawn.
 double Body::Zoom() const
 {
-	return max(zoom, 0.);
+	return max(zoom, 0.f);
+}
+
+
+
+// Check if this object is marked for removal from the game.
+bool Body::ShouldBeRemoved() const
+{
+	return shouldBeRemoved;
 }
 
 
@@ -202,7 +198,7 @@ void Body::LoadSprite(const DataNode &node)
 			delay = child.Value(1);
 		else if(child.Token(0) == "start frame" && child.Size() >= 2)
 		{
-			frameOffset += child.Value(1);
+			frameOffset += static_cast<float>(child.Value(1));
 			startAtZero = true;
 		}
 		else if(child.Token(0) == "random start frame")
@@ -222,12 +218,12 @@ void Body::LoadSprite(const DataNode &node)
 
 
 // Save the sprite specification, including all animation attributes.
-void Body::SaveSprite(DataWriter &out) const
+void Body::SaveSprite(DataWriter &out, const string &tag) const
 {
 	if(!sprite)
 		return;
 	
-	out.Write("sprite", sprite->Name());
+	out.Write(tag, sprite->Name());
 	out.BeginChild();
 	{
 		if(frameRate != static_cast<float>(2. / 60.))
@@ -265,17 +261,17 @@ void Body::SetSwizzle(int swizzle)
 
 // Set the frame rate of the sprite. This is used for objects that just specify
 // a sprite instead of a full animation data structure.
-void Body::SetFrameRate(double framesPerSecond)
+void Body::SetFrameRate(float framesPerSecond)
 {
-	frameRate = framesPerSecond / 60.;
+	frameRate = framesPerSecond / 60.f;
 }
 
 
 
 // Add the given amount to the frame rate.
-void Body::AddFrameRate(double framesPerSecond)
+void Body::AddFrameRate(float framesPerSecond)
 {
-	frameRate += framesPerSecond / 60.;
+	frameRate += framesPerSecond / 60.f;
 }
 
 
@@ -287,8 +283,24 @@ void Body::PauseAnimation()
 
 
 
+// Mark this object to be removed from the game.
+void Body::MarkForRemoval()
+{
+	shouldBeRemoved = true;
+}
+
+
+
+// Mark this object to not be removed from the game.
+void Body::UnmarkForRemoval()
+{
+	shouldBeRemoved = false;
+}
+
+
+
 // Set the current time step.
-void Body::SetStep(int step, bool isHighDPI) const
+void Body::SetStep(int step) const
 {
 	// If the animation is paused, reduce the step by however many frames it has
 	// been paused for.
@@ -297,26 +309,29 @@ void Body::SetStep(int step, bool isHighDPI) const
 	// If the step is negative or there is no sprite, do nothing. This updates
 	// and caches the mask and the frame so that if further queries are made at
 	// this same time step, we don't need to redo the calculations.
-	if(step < 0 || !sprite || !sprite->Frames() || (step == currentStep && isHighDPI == currentHighDPI))
+	if(step == currentStep || step < 0 || !sprite || !sprite->Frames())
 		return;
 	currentStep = step;
-	currentHighDPI = isHighDPI;
 	
 	// If the sprite only has one frame, no need to animate anything.
-	int frames = sprite->Frames();
-	if(frames <= 1)
+	float frames = sprite->Frames();
+	if(frames <= 1.f)
 	{
-		frame.first = sprite->Texture(0, isHighDPI);
-		activeIndex = 0;
+		frame = 0.f;
 		return;
 	}
+	float lastFrame = frames - 1.f;
+	// This is the number of frames per full cycle. If rewinding, a full cycle
+	// includes the first and last frames once and every other frame twice.
+	float cycle = (rewind ? 2.f * lastFrame : frames) + delay;
 	
 	// If this is the very first step, fill in some values that we could not set
 	// until we knew the sprite's frame count and the starting step.
 	if(randomize)
 	{
 		randomize = false;
-		frameOffset += Random::Int(sprite->Frames() + delay);
+		// The random offset can be a fractional frame.
+		frameOffset += static_cast<float>(Random::Real()) * cycle;
 	}
 	else if(startAtZero)
 	{
@@ -327,70 +342,28 @@ void Body::SetStep(int step, bool isHighDPI) const
 	
 	// Figure out what fraction of the way in between frames we are. Avoid any
 	// possible floating-point glitches that might result in a negative frame.
-	float frameF = max(0.f, frameRate * step + frameOffset);
-	frame.fade = modf(frameF, &frameF);
-	step = frameF;
+	frame = max(0.f, frameRate * step + frameOffset);
+	// If repeating, wrap the frame index by the total cycle time.
+	if(repeat)
+		frame = fmod(frame, cycle);
 	
-	int firstIndex = 0;
-	int secondIndex = 0;
 	if(!rewind)
 	{
-		// This is not a rewinding sprite. It is either playing once through, or
-		// looping from start to end over and over.
-		if(!repeat && step >= frames - 1)
+		// If not repeating, frame should never go higher than the index of the
+		// final frame.
+		if(!repeat)
+			frame = min(frame, lastFrame);
+		else if(frame >= frames)
 		{
-			// If the sprite is not repeating and it has reached the end, just
-			// stay on the last frame.
-			firstIndex = frames - 1;
-			frame.fade = 0.f;
-		}
-		else
-		{
-			// Figure out which animation step we're on. If the sprite has a
-			// delay and all the non-delay frames have been shown, show the
-			// first frame until the delay is over.
-			step %= (frames + delay);
-			if(step >= frames)
-				frame.fade = 0.f;
-			else
-			{
-				// The Sprite class automatically wraps a step that is out of
-				// bounds, so if steps == frames, (steps + 1) maps to frame 0.
-				firstIndex = step;
-				secondIndex = step + 1;
-			}
+			// If we're in the delay portion of the loop, set the frame to 0.
+			frame = 0.f;
 		}
 	}
-	else
+	else if(frame >= lastFrame)
 	{
-		// This sprite "rewinds" - that is, it plays forward, then in reverse,
-		// then possibly delays for some number of frames, then may repeat.
-		if(!repeat && step >= 2 * (frames - 1))
-			frame.fade = 0.f;
-		else
-		{
-			// The first and last frame only get shown for one step. All the
-			// others get shown twice, and a delay may also be added, so:
-			step %= 2 * (frames - 1) + delay;
-			if(step < frames - 1)
-			{
-				firstIndex = step;
-				secondIndex = step + 1;
-			}
-			else if(step < 2 * (frames - 1))
-			{
-				step = 2 * (frames - 1) - step;
-				firstIndex = step;
-				secondIndex = step - 1;
-			}
-			else
-				frame.fade = 0.f;
-		}
+		// In rewind mode, once you get to the last frame, count backwards.
+		// Regardless of whether we're repeating, if the frame count gets to
+		// be less than 0, clamp it to 0.
+		frame = max(0.f, lastFrame * 2.f - frame);
 	}
-	// Cache the frame and mask info so as long as the step stays the same, none
-	// of the above calculations need to be redone. This is important for objects
-	// whose masks may be queried many times for collision tests.
-	frame.first = sprite->Texture(firstIndex, isHighDPI);
-	frame.second = sprite->Texture(secondIndex, isHighDPI);
-	activeIndex = (frame.fade > .5f ? secondIndex : firstIndex);
 }
