@@ -15,29 +15,35 @@ PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 
 #include "AI.h"
 #include "AsteroidField.h"
+#include "BatchDrawList.h"
 #include "CollisionSet.h"
+#include "Command.h"
 #include "DrawList.h"
 #include "EscortDisplay.h"
-#include "Flotsam.h"
 #include "Information.h"
-#include "PlanetLabel.h"
 #include "Point.h"
-#include "Projectile.h"
 #include "Radar.h"
 #include "Rectangle.h"
-#include "Ship.h"
-#include "ShipEvent.h"
 
 #include <condition_variable>
 #include <list>
 #include <map>
 #include <memory>
 #include <thread>
+#include <utility>
 #include <vector>
 
+class Flotsam;
 class Government;
+class NPC;
 class Outfit;
+class PlanetLabel;
 class PlayerInfo;
+class Projectile;
+class Ship;
+class ShipEvent;
+class Sprite;
+class Visual;
 
 
 
@@ -55,6 +61,8 @@ public:
 	
 	// Place all the player's ships, and "enter" the system the player is in.
 	void Place();
+	// Place NPCs spawned by a mission that offers when the player is not landed.
+	void Place(const std::list<NPC> &npcs, std::shared_ptr<Ship> flagship = nullptr);
 	
 	// Wait for the previous calculations (if any) to be done.
 	void Wait();
@@ -65,7 +73,8 @@ public:
 	void Go();
 	
 	// Get any special events that happened in this step.
-	const std::list<ShipEvent> &Events() const;
+	// MainPanel::Step will clear this list.
+	std::list<ShipEvent> &Events();
 	
 	// Draw a frame.
 	void Draw() const;
@@ -81,6 +90,22 @@ private:
 	
 	void ThreadEntryPoint();
 	void CalculateStep();
+	
+	void MoveShip(const std::shared_ptr<Ship> &ship);
+	
+	void SpawnFleets();
+	void SpawnPersons();
+	void SendHails();
+	void HandleMouseClicks();
+	
+	void FillCollisionSets();
+	
+	void DoCollisions(Projectile &projectile);
+	void DoCollection(Flotsam &flotsam);
+	void DoScanning(const std::shared_ptr<Ship> &ship);
+	
+	void FillRadar();
+	
 	void AddSprites(const Ship &ship);
 	
 	void DoGrudge(const std::shared_ptr<Ship> &target, const Government *attacker);
@@ -93,6 +118,7 @@ private:
 		Angle angle;
 		double radius;
 		int type;
+		int count;
 	};
 	
 	class Status {
@@ -111,6 +137,21 @@ private:
 private:
 	PlayerInfo &player;
 	
+	std::list<std::shared_ptr<Ship>> ships;
+	std::vector<Projectile> projectiles;
+	std::list<std::shared_ptr<Flotsam>> flotsam;
+	std::vector<Visual> visuals;
+	AsteroidField asteroids;
+	
+	// New objects created within the latest step:
+	std::list<std::shared_ptr<Ship>> newShips;
+	std::vector<Projectile> newProjectiles;
+	std::list<std::shared_ptr<Flotsam>> newFlotsam;
+	std::vector<Visual> newVisuals;
+	
+	// Track which ships currently have anti-missiles ready to fire.
+	std::vector<Ship *> hasAntiMissile;
+	
 	AI ai;
 	
 	std::thread calcThread;
@@ -122,6 +163,7 @@ private:
 	bool terminate = false;
 	bool wasActive = false;
 	DrawList draw[2];
+	BatchDrawList batchDraw[2];
 	Radar radar[2];
 	// Viewport position and velocity.
 	Point center;
@@ -129,8 +171,9 @@ private:
 	// Other information to display.
 	Information info;
 	std::vector<Target> targets;
-	Point targetAngle;
+	Point targetVector;
 	Point targetUnit;
+	int targetSwizzle = -1;
 	EscortDisplay escorts;
 	std::vector<Status> statuses;
 	std::vector<PlanetLabel> labels;
@@ -139,25 +182,17 @@ private:
 	const System *jumpInProgress[2] = {nullptr, nullptr};
 	const Sprite *highlightSprite = nullptr;
 	Point highlightUnit;
-	int highlightFrame = 0;
+	float highlightFrame = 0.f;
 	
 	int step = 0;
 	
-	std::list<std::shared_ptr<Ship>> ships;
-	std::list<Projectile> projectiles;
-	std::list<std::shared_ptr<Flotsam>> flotsam;
-	std::list<Effect> effects;
-	
 	std::list<ShipEvent> eventQueue;
 	std::list<ShipEvent> events;
 	// Keep track of who has asked for help in fighting whom.
 	std::map<const Government *, std::weak_ptr<const Ship>> grudge;
 	int grudgeTime = 0;
 	
-	AsteroidField asteroids;
-	
 	CollisionSet shipCollisions;
-	CollisionSet cloakedCollisions;
 	
 	int alarmTime = 0;
 	double flash = 0.;
@@ -170,6 +205,7 @@ private:
 	bool hasShift = false;
 	bool hasControl = false;
 	bool isRightClick = false;
+	bool isRadarClick = false;
 	Point clickPoint;
 	Rectangle clickBox;
 	int groupSelect = -1;
