@@ -26,9 +26,117 @@ PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 
 #include <algorithm>
 #include <cmath>
+#include <iterator>
 
 using namespace std;
 
+namespace {
+	// Generate an offset magnitude that will sample from an annulus (planets)
+	// or a circle (systems without inhabited planets).
+	double OffsetFrom(pair<Point, double> &center)
+	{
+		// If the center has a radius, then position ships further away.
+		double minimumOffset = center.second ? 1. : 0.;
+		// Since it is sensible that ships would be nearer to the object of
+		// interest on average, do not apply the sqrt(rand) correction.
+		return (Random::Real() + minimumOffset) * 400. + 2. * center.second;
+	}
+	
+	// Construct a list of all outfits for sale in this system and its linked neighbors.
+	Sale<Outfit> GetOutfitsForSale(const System *here)
+	{
+		auto outfits = Sale<Outfit>();
+		if(here)
+		{
+			for(const StellarObject &object : here->Objects())
+			{
+				const Planet *planet = object.GetPlanet();
+				if(planet && planet->HasOutfitter())
+					outfits.Add(planet->Outfitter());
+			}
+		}
+		return outfits;
+	}
+	
+	// Construct a list of varying numbers of outfits that were either specified for
+	// this fleet directly, or are sold in this system or its linked neighbors.
+	vector<const Outfit *> OutfitChoices(const set<const Sale<Outfit> *> &outfitters, const System *hub, int maxSize)
+	{
+		auto outfits = vector<const Outfit *>();
+		if(maxSize > 0)
+		{
+			auto choices = Sale<Outfit>();
+			// If no outfits were directly specified, choose from those sold nearby.
+			if(outfitters.empty() && hub)
+			{
+				choices = GetOutfitsForSale(hub);
+				for(const System *other : hub->Links())
+					choices.Add(GetOutfitsForSale(other));
+			}
+			else
+				for(const auto outfitter : outfitters)
+					choices.Add(*outfitter);
+			
+			if(!choices.empty())
+			{
+				for(const auto outfit : choices)
+				{
+					double mass = outfit->Mass();
+					// Avoid free outfits, massless outfits, and those too large to fit.
+					if(mass > 0. && mass < maxSize && outfit->Cost() > 0)
+						outfits.push_back(outfit);
+				}
+			}
+		}
+		// Sort this list of choices ascending by mass, so it can be easily trimmed to just
+		// the outfits that fit as the ship's free space decreases.
+		sort(outfits.begin(), outfits.end(), [](const Outfit *a, const Outfit *b){ return a->Mass() < b->Mass(); });
+		return outfits;
+	}
+	
+	// Add a random commodity from the list to the ship's cargo.
+	void AddRandomCommodity(Ship &ship, int freeSpace, const vector<string> &commodities)
+	{
+		int index = Random::Int(GameData::Commodities().size());
+		if(!commodities.empty())
+		{
+			// If a list of possible commodities was given, pick one of them at
+			// random and then double-check that it's a valid commodity name.
+			const string &name = commodities[Random::Int(commodities.size())];
+			for(const auto &it : GameData::Commodities())
+				if(it.name == name)
+				{
+					index = &it - &GameData::Commodities().front();
+					break;
+				}
+		}
+		
+		const Trade::Commodity &commodity = GameData::Commodities()[index];
+		int amount = Random::Int(freeSpace) + 1;
+		ship.Cargo().Add(commodity.name, amount);
+	}
+	
+	// Add a random outfit from the list to the ship's cargo.
+	void AddRandomOutfit(Ship &ship, int freeSpace, const vector<const Outfit *> &outfits)
+	{
+		if(outfits.empty())
+			return;
+		int index = Random::Int(outfits.size());
+		const Outfit *picked = outfits[index];
+		int maxQuantity = floor(static_cast<double>(freeSpace) / picked->Mass());
+		int amount = Random::Int(maxQuantity) + 1;
+		ship.Cargo().Add(picked, amount);
+	}
+}
+
+
+
+// Construct and Load() at the same time.
+Fleet::Fleet(const DataNode &node)
+{
+	Load(node);
+}
+
 
 
 void Fleet::Load(const DataNode &node)
@@ -42,12 +150,19 @@ void Fleet::Load(const DataNode &node)
 	
 	for(const DataNode &child : node)
 	{
-		const string &key = child.Token(0);
+		// The "add" and "remove" keywords should never be alone on a line, and
+		// are only valid with "variant" or "personality" definitions.
+		bool add = (child.Token(0) == "add");
+		bool remove = (child.Token(0) == "remove");
 		bool hasValue = (child.Size() >= 2);
-		// Special case: "add/remove variant" means to add or remove a variant without
-		// clearing the rest of the existing definition.
-		bool add = (key == "add" && hasValue && child.Token(1) == "variant");
-		bool remove = (key == "remove" && hasValue && child.Token(1) == "variant");
+		if((add || remove) && (!hasValue || (child.Token(1) != "variant" && child.Token(1) != "personality")))
+		{	
+			child.PrintTrace("Skipping invalid \"" + child.Token(0) + "\" tag:");
+			continue;
+		}
+		
+		// If this line is an add or remove, the key is the token at index 1.
+		const string &key = child.Token(add || remove);
 		
 		if(key == "government" && hasValue)
 			government = GameData::Governments().Get(child.Token(1));
@@ -63,9 +178,15 @@ void Fleet::Load(const DataNode &node)
 			for(int i = 1; i < child.Size(); ++i)
 				commodities.push_back(child.Token(i));
 		}
+		else if(key == "outfitters" && hasValue)
+		{
+			outfitters.clear();
+			for(int i = 1; i < child.Size(); ++i)
+				outfitters.insert(GameData::Outfitters().Get(child.Token(i)));
+		}
 		else if(key == "personality")
 			personality.Load(child);
-		else if(key == "variant" || add)
+		else if(key == "variant" && !remove)
 		{
 			if(resetVariants && !add)
 			{
@@ -76,13 +197,12 @@ void Fleet::Load(const DataNode &node)
 			variants.emplace_back(child);
 			total += variants.back().weight;
 		}
-		else if(remove)
+		else if(key == "variant")
 		{
 			// If given a full ship definition of one of this fleet's variant members, remove the variant.
 			bool didRemove = false;
+			Variant toRemove(child);
 			for(auto it = variants.begin(); it != variants.end(); ++it)
-			{
-				Variant toRemove = Variant(child);
 				if(toRemove.ships.size() == it->ships.size() &&
 					is_permutation(it->ships.begin(), it->ships.end(), toRemove.ships.begin()))
 				{
@@ -91,7 +211,7 @@ void Fleet::Load(const DataNode &node)
 					didRemove = true;
 					break;
 				}
-			}
+			
 			if(!didRemove)
 				child.PrintTrace("Did not find matching variant for specified operation:");
 		}
@@ -109,7 +229,7 @@ const Government *Fleet::GetGovernment() const
 }
 
 
-
+// Choose a fleet to be created during flight, and have it enter the system via jump or planetary departure.
 void Fleet::Enter(const System &system, list<shared_ptr<Ship>> &ships, const Planet *planet) const
 {
 	if(!total || variants.empty())
@@ -120,14 +240,14 @@ void Fleet::Enter(const System &system, list<shared_ptr<Ship>> &ships, const Pla
 	if(variant.ships.empty())
 		return;
 	
-	// Figure out what system the ship is starting in, where it is going, and
+	// Figure out what system the fleet is starting in, where it is going, and
 	// what position it should start from in the system.
 	const System *source = &system;
 	const System *target = &system;
 	Point position;
-	double radius = 0.;
+	double radius = 1000.;
 	
-	// Only pick a random entry point for this ship if a source planet was not specified.
+	// Only pick a random entry point for this fleet if a source planet was not specified.
 	if(!planet)
 	{
 		// Where this fleet can come from depends on whether it is friendly to any
@@ -142,7 +262,10 @@ void Fleet::Enter(const System &system, list<shared_ptr<Ship>> &ships, const Pla
 		for(const Ship *ship : variant.ships)
 		{
 			if(ship->Attributes().Get("jump drive"))
+			{
 				hasJump = true;
+				break;
+			}
 			if(ship->Attributes().Get("hyperdrive"))
 				hasHyper = true;
 		}
@@ -195,33 +318,53 @@ void Fleet::Enter(const System &system, list<shared_ptr<Ship>> &ships, const Pla
 			if(!linkVector.empty())
 				target = linkVector[Random::Int(linkVector.size())];
 		}
+		// We are entering this system via hyperspace, not taking off from a planet.
 		else
-		{
-			// We are entering this system via hyperspace, not taking off from a planet.
-			radius = 1000.;
 			source = linkVector[choice];
-		}
 	}
 	
-	// Find the stellar object for the given planet, and place the ships there.
+	auto placed = Instantiate(variant);
+	// Carry all ships that can be carried, as they don't need to be positioned
+	// or checked to see if they can access a particular planet.
+	for(auto &ship : placed)
+		PlaceFighter(ship, placed);
+	
+	// Find the stellar object for this planet, and place the ships there.
 	if(planet)
 	{
-		for(const StellarObject &object : system.Objects())
-			if(object.GetPlanet() == planet)
-			{
-				position = object.Position();
-				radius = object.Radius();
-				break;
-			}
+		const StellarObject *object = system.FindStellar(planet);
+		if(!object)
+		{
+			// Log this error.
+			Files::LogError("Fleet::Enter: Unable to find valid stellar object for planet \""
+				+ planet->TrueName() + "\" in system \"" + system.Name() + "\"");
+			return;
+		}
+		// To take off from the planet, all non-carried ships must be able to access it.
+		else if(planet->IsUnrestricted() || all_of(placed.cbegin(), placed.cend(), [&](const shared_ptr<Ship> &ship)
+				{ return ship->GetParent() || planet->IsAccessible(ship.get()); }))
+		{
+			position = object->Position();
+			radius = object->Radius();
+		}
+		// The chosen planet could not be departed from by all ships in the variant.
+		else
+		{
+			// If there are no departure paths, then there are no arrival paths either.
+			if(source == target)
+				return;
+			// Otherwise, have the fleet arrive here from the target system.
+			std::swap(source, target);
+			planet = nullptr;
+		}
 	}
 	
 	// Place all the ships in the chosen fleet variant.
 	shared_ptr<Ship> flagship;
-	vector<shared_ptr<Ship>> placed = Instantiate(variant);
 	for(shared_ptr<Ship> &ship : placed)
 	{
-		// If this is a fighter and someone can carry it, no need to position it.
-		if(PlaceFighter(ship, placed))
+		// If this is a carried fighter, no need to position it.
+		if(ship->GetParent())
 			continue;
 		
 		Angle angle = Angle::Random(360.);
@@ -252,7 +395,8 @@ void Fleet::Enter(const System &system, list<shared_ptr<Ship>> &ships, const Pla
 
 
 
-// Place a fleet in the given system, already "in action."
+// Place one of the variants in the given system, already "in action." If the carried flag is set,
+// only uncarried ships will be added to the list (as any carriables will be stored in bays).
 void Fleet::Place(const System &system, list<shared_ptr<Ship>> &ships, bool carried) const
 {
 	if(!total || variants.empty())
@@ -264,7 +408,7 @@ void Fleet::Place(const System &system, list<shared_ptr<Ship>> &ships, bool carr
 		return;
 	
 	// Determine where the fleet is going to or coming from.
-	Point center = ChooseCenter(system);
+	auto center = ChooseCenter(system);
 	
 	// Place all the ships in the chosen fleet variant.
 	shared_ptr<Ship> flagship;
@@ -276,7 +420,7 @@ void Fleet::Place(const System &system, list<shared_ptr<Ship>> &ships, bool carr
 			continue;
 		
 		Angle angle = Angle::Random();
-		Point pos = center + Angle::Random().Unit() * Random::Real() * 400.;
+		Point pos = center.first + Angle::Random().Unit() * OffsetFrom(center);
 		double velocity = Random::Real() * ship->MaxVelocity();
 		
 		ships.push_front(ship);
@@ -295,36 +439,41 @@ void Fleet::Place(const System &system, list<shared_ptr<Ship>> &ships, bool carr
 
 
 // Do the randomization to make a ship enter or be in the given system.
-void Fleet::Enter(const System &system, Ship &ship)
+const System *Fleet::Enter(const System &system, Ship &ship, const System *source)
 {
-	if(system.Links().empty())
+	if(system.Links().empty() || (source && !system.Links().count(source)))
 	{
 		Place(system, ship);
-		return;
+		return &system;
 	}
 	
 	// Choose which system this ship is coming from.
-	int choice = Random::Int(system.Links().size());
-	set<const System *>::const_iterator it = system.Links().begin();
-	while(choice--)
-		++it;
+	if(!source)
+	{
+		auto it = system.Links().cbegin();
+		advance(it, Random::Int(system.Links().size()));
+		source = *it;
+	}
 	
 	Angle angle = Angle::Random();
 	Point pos = angle.Unit() * Random::Real() * 1000.;
 	
 	ship.Place(pos, angle.Unit(), angle);
-	ship.SetSystem(*it);
+	ship.SetSystem(source);
 	ship.SetTargetSystem(&system);
+	
+	return source;
 }
 
 
 
 void Fleet::Place(const System &system, Ship &ship)
 {
-	// Move out a random distance from that object, facing toward it or away.
-	Point pos = ChooseCenter(system) + Angle::Random().Unit() * Random::Real() * 400.;
+	// Choose a random inhabited object in the system to spawn around.
+	auto center = ChooseCenter(system);
+	Point pos = center.first + Angle::Random().Unit() * OffsetFrom(center);
 	
-	double velocity = Random::Real() * ship.MaxVelocity();
+	double velocity = ship.IsDisabled() ? 0. : Random::Real() * ship.MaxVelocity();
 	
 	ship.SetSystem(&system);
 	Angle angle = Angle::Random();
@@ -332,7 +481,7 @@ void Fleet::Place(const System &system, Ship &ship)
 }
 
 
-	
+
 int64_t Fleet::Strength() const
 {
 	int64_t sum = 0;
@@ -379,15 +528,17 @@ const Fleet::Variant &Fleet::ChooseVariant() const
 
 
 
-Point Fleet::ChooseCenter(const System &system)
+// Obtain a positional reference and the radius of the object at that position (e.g. a planet).
+// Spaceport status can be modified during normal gameplay, so this information is not cached.
+pair<Point, double> Fleet::ChooseCenter(const System &system)
 {
-	vector<Point> centers;
+	auto centers = vector<pair<Point, double>>();
 	for(const StellarObject &object : system.Objects())
 		if(object.GetPlanet() && object.GetPlanet()->HasSpaceport())
-			centers.push_back(object.Position());
+			centers.emplace_back(object.Position(), object.Radius());
 	
 	if(centers.empty())
-		return Point();
+		return {Point(), 0.};
 	return centers[Random::Int(centers.size())];
 }
 
@@ -434,31 +585,42 @@ bool Fleet::PlaceFighter(shared_ptr<Ship> fighter, vector<shared_ptr<Ship>> &pla
 
 
 
+// Choose the cargo associated with this ship in the fleet.
+// If outfits were specified, but not commodities, do not pick commodities.
+// If commodities were specified, but not outfits, do not pick outfits.
+// If neither or both were specified, choose commodities more often..
 void Fleet::SetCargo(Ship *ship) const
 {
+	const bool canChooseOutfits = commodities.empty() || !outfitters.empty();
+	const bool canChooseCommodities = outfitters.empty() || !commodities.empty();
+	// Populate the possible outfits that may be chosen.
+	int free = ship->Cargo().Free();
+	auto outfits = OutfitChoices(outfitters, ship->GetSystem(), free);
+	
+	// Choose random outfits or commodities to transport.
 	for(int i = 0; i < cargo; ++i)
 	{
-		int free = ship->Cargo().Free();
 		if(!free)
 			break;
+		// Remove any outfits that do not fit into remaining cargo.
+		if(canChooseOutfits && !outfits.empty())
+			outfits.erase(remove_if(outfits.begin(), outfits.end(),
+					[&free](const Outfit *a) { return a->Mass() > free; }),
+				outfits.end());
 		
-		int index = Random::Int(GameData::Commodities().size());
-		if(commodities.size())
+		if(canChooseCommodities && canChooseOutfits)
 		{
-			// If a list of possible commodities was given, pick one of them at
-			// random and then double-check that it's a valid commodity name.
-			const string &name = commodities[Random::Int(commodities.size())];
-			for(const auto &it : GameData::Commodities())
-				if(it.name == name)
-				{
-					index = &it - &GameData::Commodities().front();
-					break;
-				}
+			if(Random::Real() < .8)
+				AddRandomCommodity(*ship, free, commodities);
+			else
+				AddRandomOutfit(*ship, free, outfits);
 		}
+		else if(canChooseCommodities)
+			AddRandomCommodity(*ship, free, commodities);
+		else
+			AddRandomOutfit(*ship, free, outfits);
 		
-		const Trade::Commodity &commodity = GameData::Commodities()[index];
-		int amount = Random::Int(free) + 1;
-		ship->Cargo().Add(commodity.name, amount);
+		free = ship->Cargo().Free();
 	}
 	int extraCrew = ship->Attributes().Get("bunks") - ship->RequiredCrew();
 	if(extraCrew > 0)
