@@ -25,9 +25,9 @@ PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 using namespace std;
 
 namespace {
-	static bool showUnderlines = false;
+	bool showUnderlines = false;
 	
-	static const char *vertexCode =
+	const char *vertexCode =
 		// "scale" maps pixel coordinates to GL coordinates (-1 to 1).
 		"uniform vec2 scale;\n"
 		// The (x, y) coordinates of the top left corner of the glyph.
@@ -50,7 +50,7 @@ namespace {
 		"  gl_Position = vec4((aspect * vert.x + position.x) * scale.x, (vert.y + position.y) * scale.y, 0, 1);\n"
 		"}\n";
 	
-	static const char *fragmentCode =
+	const char *fragmentCode =
 		// The user must supply a texture and a color (white by default).
 		"uniform sampler2D tex;\n"
 		"uniform vec4 color = vec4(1, 1, 1, 1);\n"
@@ -66,7 +66,7 @@ namespace {
 		"  finalColor = texture(tex, texCoord).a * color;\n"
 		"}\n";
 	
-	static const int KERN = 2;
+	const int KERN = 2;
 }
 
 
@@ -90,15 +90,13 @@ Font::Font(const string &imagePath)
 void Font::Load(const string &imagePath)
 {
 	// Load the texture.
-	ImageBuffer *image = ImageBuffer::Read(imagePath);
-	if(!image)
+	ImageBuffer image;
+	if(!image.Read(imagePath))
 		return;
 	
 	LoadTexture(image);
 	CalculateAdvances(image);
-	SetUpShader(image->Width() / GLYPHS, image->Height());
-	
-	delete image;
+	SetUpShader(image.Width() / GLYPHS, image.Height());
 }
 
 
@@ -113,7 +111,6 @@ void Font::Draw(const string &str, const Point &point, const Color &color) const
 void Font::DrawAliased(const string &str, double x, double y, const Color &color) const
 {
 	glUseProgram(shader.Object());
-	glActiveTexture(GL_TEXTURE0);
 	glBindTexture(GL_TEXTURE_2D, texture);
 	glBindVertexArray(vao);
 	
@@ -175,6 +172,9 @@ void Font::DrawAliased(const string &str, double x, double y, const Color &color
 		
 		previous = glyph;
 	}
+	
+	glBindVertexArray(0);
+	glUseProgram(0);
 }
 
 
@@ -357,7 +357,7 @@ int Font::Glyph(char c, bool isAfterSpace)
 
 
 
-void Font::LoadTexture(ImageBuffer *image)
+void Font::LoadTexture(ImageBuffer &image)
 {
 	glGenTextures(1, &texture);
 	glBindTexture(GL_TEXTURE_2D, texture);
@@ -367,20 +367,20 @@ void Font::LoadTexture(ImageBuffer *image)
 	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
 	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
 	
-	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA8, image->Width(), image->Height(), 0,
-		GL_BGRA, GL_UNSIGNED_BYTE, image->Pixels());
+	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA8, image.Width(), image.Height(), 0,
+		GL_BGRA, GL_UNSIGNED_BYTE, image.Pixels());
 }
 
 
 
-void Font::CalculateAdvances(ImageBuffer *image)
+void Font::CalculateAdvances(ImageBuffer &image)
 {
 	// Get the format and size of the surface.
-	int width = image->Width() / GLYPHS;
-	height = image->Height();
+	int width = image.Width() / GLYPHS;
+	height = image.Height();
 	unsigned mask = 0xFF000000;
 	unsigned half = 0xC0000000;
-	int pitch = image->Width();
+	int pitch = image.Width();
 	
 	// advance[previous * GLYPHS + next] is the x advance for each glyph pair.
 	// There is no advance if the previous value is 0, i.e. we are at the very
@@ -391,7 +391,7 @@ void Font::CalculateAdvances(ImageBuffer *image)
 		{
 			int maxD = 0;
 			int glyphWidth = 0;
-			uint32_t *begin = reinterpret_cast<uint32_t *>(image->Pixels());
+			uint32_t *begin = image.Pixels();
 			for(int y = 0; y < height; ++y)
 			{
 				// Find the last non-empty pixel in the previous glyph.
@@ -442,6 +442,8 @@ void Font::SetUpShader(float glyphW, float glyphH)
 	
 	shader = Shader(vertexCode, fragmentCode);
 	glUseProgram(shader.Object());
+	glUniform1i(shader.Uniform("tex"), 0);
+	glUseProgram(0);
 	
 	// Create the VAO and VBO.
 	glGenVertexArrays(1, &vao);
@@ -466,13 +468,13 @@ void Font::SetUpShader(float glyphW, float glyphH)
 	glVertexAttribPointer(shader.Attrib("corner"), 2, GL_FLOAT, GL_FALSE,
 		4 * sizeof(GLfloat), (const GLvoid*)(2 * sizeof(GLfloat)));
 	
+	glBindBuffer(GL_ARRAY_BUFFER, 0);
+	glBindVertexArray(0);
+	
 	// We must update the screen size next time we draw.
 	screenWidth = 0;
 	screenHeight = 0;
 	
-	// The texture always comes from texture unit 0.
-	glUniform1ui(shader.Uniform("tex"), 0);
-
 	colorI = shader.Uniform("color");
 	scaleI = shader.Uniform("scale");
 	glyphI = shader.Uniform("glyph");
