@@ -102,6 +102,7 @@ void MapOutfitterPanel::Select(int index)
 		selected = list[index];
 		selectedInfo.Update(*selected, player);
 	}
+	UpdateCache();
 }
 
 
@@ -183,7 +184,7 @@ void MapOutfitterPanel::DrawItems()
 		
 		for(const Outfit *outfit : it->second)
 		{
-			string price = Format::Number(outfit->Cost()) + " credits";
+			string price = Format::Credits(outfit->Cost()) + " credits";
 			
 			string info;
 			if(outfit->Get("installable") < 0.)
