@@ -61,6 +61,24 @@ namespace {
 		// Control will never reach here, but to satisfy the compiler:
 		return nullptr;
 	}
+	
+	// If a source, destination, waypoint, or stopover supplies more than one explicit choice
+	// or a mixture of explicit choice and location filter, print a warning.
+	void ParseMixedSpecificity(const DataNode &node, string &&kind, int expected)
+	{
+		if(node.Size() >= expected + 1)
+			node.PrintTrace("Warning: use a location filter to choose from multiple " + kind + "s:");
+		if(node.HasChildren())
+			node.PrintTrace("Warning: location filter ignored due to use of explicit " + kind + ":");
+	}
+}
+
+
+
+// Construct and Load() at the same time.
+Mission::Mission(const DataNode &node)
+{
+	Load(node);
 }
 
 
@@ -68,10 +86,22 @@ namespace {
 // Load a mission, either from the game data or from a saved game.
 void Mission::Load(const DataNode &node)
 {
-	if(node.Size() >= 2)
-		name = node.Token(1);
-	else
-		name = "Unnamed Mission";
+	// All missions need a name.
+	if(node.Size() < 2)
+	{
+		node.PrintTrace("No name specified for mission:");
+		return;
+	}
+	// If a mission object is "loaded" twice, that is most likely an error (e.g.
+	// due to a plugin containing a mission with the same name as the base game
+	// or another plugin). This class is not designed to allow merging or
+	// overriding of mission data from two different definitions.
+	if(!name.empty())
+	{
+		node.PrintTrace("Duplicate definition of mission:");
+		return;
+	}
+	name = node.Token(1);
 	
 	for(const DataNode &child : node)
 	{
@@ -103,17 +133,10 @@ void Mission::Load(const DataNode &node)
 			
 			for(const DataNode &grand : child)
 			{
-				if(grand.Token(0) == "illegal" && grand.Size() == 2)
-					illegalCargoFine = grand.Value(1);
-				else if(grand.Token(0) == "illegal" && grand.Size() == 3)
-				{
-					illegalCargoFine = grand.Value(1);
-					illegalCargoMessage = grand.Token(2);
-				}
-				else if(grand.Token(0) == "stealth")
-					failIfDiscovered = true;
-				else
+				if(!ParseContraband(grand))
 					grand.PrintTrace("Skipping unrecognized attribute:");
+				else
+					grand.PrintTrace("Warning: \"stealth\" and \"illegal\" are now mission-level properties:");
 			}
 		}
 		else if(child.Token(0) == "passengers" && child.Size() >= 2)
@@ -124,6 +147,11 @@ void Mission::Load(const DataNode &node)
 			if(child.Size() >= 4)
 				passengerProb = child.Value(3);
 		}
+		else if(ParseContraband(child))
+		{
+			// This was an "illegal" or "stealth" entry. It has already been
+			// parsed, so nothing more needs to be done here.
+		}
 		else if(child.Token(0) == "invisible")
 			isVisible = false;
 		else if(child.Token(0) == "priority")
@@ -163,39 +191,50 @@ void Mission::Load(const DataNode &node)
 				child.PrintTrace("Skipping unrecognized attribute:");
 		}
 		else if(child.Token(0) == "source" && child.Size() >= 2)
+		{
 			source = GameData::Planets().Get(child.Token(1));
+			ParseMixedSpecificity(child, "planet", 2);
+		}
 		else if(child.Token(0) == "source")
 			sourceFilter.Load(child);
 		else if(child.Token(0) == "destination" && child.Size() == 2)
+		{
 			destination = GameData::Planets().Get(child.Token(1));
+			ParseMixedSpecificity(child, "planet", 2);
+		}
 		else if(child.Token(0) == "destination")
 			destinationFilter.Load(child);
 		else if(child.Token(0) == "waypoint" && child.Size() >= 2)
-			waypoints.insert(GameData::Systems().Get(child.Token(1)));
-		else if(child.Token(0) == "waypoint")
 		{
-			waypointFilters.emplace_back();
-			waypointFilters.back().Load(child);
+			bool visited = child.Size() >= 3 && child.Token(2) == "visited";
+			set<const System *> &set = visited ? visitedWaypoints : waypoints;
+			set.insert(GameData::Systems().Get(child.Token(1)));
+			ParseMixedSpecificity(child, "system", 2 + visited);
 		}
+		else if(child.Token(0) == "waypoint" && child.HasChildren())
+			waypointFilters.emplace_back(child);
 		else if(child.Token(0) == "stopover" && child.Size() >= 2)
-			stopovers.insert(GameData::Planets().Get(child.Token(1)));
-		else if(child.Token(0) == "stopover")
 		{
-			stopoverFilters.emplace_back();
-			stopoverFilters.back().Load(child);
+			bool visited = child.Size() >= 3 && child.Token(2) == "visited";
+			set<const Planet *> &set = visited ? visitedStopovers : stopovers;
+			set.insert(GameData::Planets().Get(child.Token(1)));
+			ParseMixedSpecificity(child, "planet", 2 + visited);
 		}
+		else if(child.Token(0) == "stopover" && child.HasChildren())
+			stopoverFilters.emplace_back(child);
 		else if(child.Token(0) == "npc")
-		{
-			npcs.push_back(NPC());
-			npcs.back().Load(child);
-		}
+			npcs.emplace_back(child);
 		else if(child.Token(0) == "on" && child.Size() >= 2 && child.Token(1) == "enter")
 		{
-			const System *system = nullptr;
+			// "on enter" nodes may either name a specific system or use a LocationFilter
+			// to control the triggering system.
 			if(child.Size() >= 3)
-				system = GameData::Systems().Get(child.Token(2));
-			MissionAction &action = onEnter[system];
-			action.Load(child, name);
+			{
+				MissionAction &action = onEnter[GameData::Systems().Get(child.Token(2))];
+				action.Load(child, name);
+			}
+			else
+				genericOnEnter.emplace_back(child, name);
 		}
 		else if(child.Token(0) == "on" && child.Size() >= 2)
 		{
@@ -240,27 +279,13 @@ void Mission::Save(DataWriter &out, const string &tag) const
 		if(deadline)
 			out.Write("deadline", deadline.Day(), deadline.Month(), deadline.Year());
 		if(cargoSize)
-		{
 			out.Write("cargo", cargo, cargoSize);
-			if(illegalCargoFine)
-			{
-				out.BeginChild();
-				{
-					out.Write("illegal", illegalCargoFine, illegalCargoMessage);
-				}
-				out.EndChild();
-			}
-			if(failIfDiscovered)
-			{
-				out.BeginChild();
-				{
-					out.Write("stealth");
-				}
-				out.EndChild();
-			}
-		}
 		if(passengers)
 			out.Write("passengers", passengers);
+		if(illegalCargoFine)
+			out.Write("illegal", illegalCargoFine, illegalCargoMessage);
+		if(failIfDiscovered)
+			out.Write("stealth");
 		if(!isVisible)
 			out.Write("invisible");
 		if(hasPriority)
@@ -320,8 +345,13 @@ void Mission::Save(DataWriter &out, const string &tag) const
 			out.Write("destination", destination->Name());
 		for(const System *system : waypoints)
 			out.Write("waypoint", system->Name());
+		for(const System *system : visitedWaypoints)
+			out.Write("waypoint", system->Name(), "visited");
+		
 		for(const Planet *planet : stopovers)
 			out.Write("stopover", planet->Name());
+		for(const Planet *planet : visitedStopovers)
+			out.Write("stopover", planet->Name(), "visited");
 		
 		for(const NPC &npc : npcs)
 			npc.Save(out);
@@ -330,9 +360,13 @@ void Mission::Save(DataWriter &out, const string &tag) const
 		// has not been received yet but must still be included in the saved game.
 		for(const auto &it : actions)
 			it.second.Save(out);
+		// Save any "on enter" actions that have not been performed.
 		for(const auto &it : onEnter)
-			if(!didEnter.count(it.first))
+			if(!didEnter.count(&it.second))
 				it.second.Save(out);
+		for(const MissionAction &action : genericOnEnter)
+			if(!didEnter.count(&action))
+				action.Save(out);
 	}
 	out.EndChild();
 }
@@ -397,20 +431,34 @@ const Planet *Mission::Destination() const
 
 
 
-set<const System *> Mission::Waypoints() const
+const set<const System *> &Mission::Waypoints() const
 {
 	return waypoints;
 }
 
 
 
-set<const Planet *> Mission::Stopovers() const
+const set<const System *> &Mission::VisitedWaypoints() const
+{
+	return visitedWaypoints;
+}
+
+
+
+const set<const Planet *> &Mission::Stopovers() const
 {
 	return stopovers;
 }
 
 
 
+const set<const Planet *> &Mission::VisitedStopovers() const
+{
+	return visitedStopovers;
+}
+
+
+
 const string &Mission::Cargo() const
 {
 	return cargo;
@@ -480,7 +528,7 @@ bool Mission::HasClearance(const Planet *planet) const
 {
 	if(clearance.empty())
 		return false;
-	if(planet == destination || stopovers.count(planet))
+	if(planet == destination || stopovers.count(planet) || visitedStopovers.count(planet))
 		return true;
 	return (!clearanceFilter.IsEmpty() && clearanceFilter.Matches(planet));
 }
@@ -506,14 +554,14 @@ bool Mission::HasFullClearance() const
 
 
 // Check if it's possible to offer or complete this mission right now.
-bool Mission::CanOffer(const PlayerInfo &player) const
+bool Mission::CanOffer(const PlayerInfo &player, const shared_ptr<Ship> &boardingShip) const
 {
 	if(location == BOARDING || location == ASSISTING)
 	{
-		if(!player.BoardingShip())
+		if(!boardingShip)
 			return false;
 		
-		if(!sourceFilter.Matches(*player.BoardingShip()))
+		if(!sourceFilter.Matches(*boardingShip))
 			return false;
 	}
 	else
@@ -539,15 +587,15 @@ bool Mission::CanOffer(const PlayerInfo &player) const
 	}
 	
 	auto it = actions.find(OFFER);
-	if(it != actions.end() && !it->second.CanBeDone(player))
+	if(it != actions.end() && !it->second.CanBeDone(player, boardingShip))
 		return false;
 	
 	it = actions.find(ACCEPT);
-	if(it != actions.end() && !it->second.CanBeDone(player))
+	if(it != actions.end() && !it->second.CanBeDone(player, boardingShip))
 		return false;
 	
 	it = actions.find(DECLINE);
-	if(it != actions.end() && !it->second.CanBeDone(player))
+	if(it != actions.end() && !it->second.CanBeDone(player, boardingShip))
 		return false;
 	
 	return true;
@@ -561,23 +609,46 @@ bool Mission::HasSpace(const PlayerInfo &player) const
 	if(player.Flagship())
 		extraCrew = player.Flagship()->Crew() - player.Flagship()->RequiredCrew();
 	return (cargoSize <= player.Cargo().Free() + player.Cargo().CommoditiesSize()
-		&& passengers <= player.Cargo().Bunks() + extraCrew);
+		&& passengers <= player.Cargo().BunksFree() + extraCrew);
+}
+
+
+
+// Check if this mission's cargo can fit entirely on the referenced ship.
+bool Mission::HasSpace(const Ship &ship) const
+{
+	return (cargoSize <= ship.Cargo().Free() && passengers <= ship.Cargo().BunksFree());
 }
 
 
 
 bool Mission::CanComplete(const PlayerInfo &player) const
 {
-	if(player.GetPlanet() != destination || !waypoints.empty() || !stopovers.empty())
+	if(player.GetPlanet() != destination)
 		return false;
 	
 	if(!toComplete.Test(player.Conditions()))
 		return false;
 	
+	return IsSatisfied(player);
+}
+
+
+
+// This function dictates whether missions on the player's map are shown in
+// bright or dim text colors.
+bool Mission::IsSatisfied(const PlayerInfo &player) const
+{
+	if(!waypoints.empty() || !stopovers.empty())
+		return false;
+	
+	// Determine if any fines or outfits that must be transferred, can.
 	auto it = actions.find(COMPLETE);
 	if(it != actions.end() && !it->second.CanBeDone(player))
 		return false;
 	
+	// NPCs which must be accompanied or evaded must be present (or not),
+	// and any needed scans, boarding, or assisting must also be completed.
 	for(const NPC &npc : npcs)
 		if(!npc.HasSucceeded(player.GetSystem()))
 			return false;
@@ -625,19 +696,32 @@ string Mission::BlockedMessage(const PlayerInfo &player)
 		return "";
 	
 	int extraCrew = 0;
-	if(player.Flagship())
-		extraCrew = player.Flagship()->Crew() - player.Flagship()->RequiredCrew();
+	const Ship *flagship = player.Flagship();
+	// You cannot fire crew in space.
+	if(flagship && player.GetPlanet())
+		extraCrew = flagship->Crew() - flagship->RequiredCrew();
 	
-	int cargoNeeded = cargoSize - (player.Cargo().Free() + player.Cargo().CommoditiesSize());
-	int bunksNeeded = passengers - (player.Cargo().Bunks() + extraCrew);
+	int cargoNeeded = cargoSize;
+	int bunksNeeded = passengers;
+	if(player.GetPlanet())
+	{
+		cargoNeeded -= (player.Cargo().Free() + player.Cargo().CommoditiesSize());
+		bunksNeeded -= (player.Cargo().BunksFree() + extraCrew);
+	}
+	else
+	{
+		// Boarding a ship, so only use the flagship's space.
+		cargoNeeded -= flagship->Cargo().Free();
+		bunksNeeded -= flagship->Cargo().BunksFree();
+	}
 	if(cargoNeeded < 0 && bunksNeeded < 0)
 		return "";
 	
 	map<string, string> subs;
 	subs["<first>"] = player.FirstName();
 	subs["<last>"] = player.LastName();
-	if(player.Flagship())
-		subs["<ship>"] = player.Flagship()->Name();
+	if(flagship)
+		subs["<ship>"] = flagship->Name();
 	
 	ostringstream out;
 	if(bunksNeeded > 0)
@@ -677,7 +761,7 @@ bool Mission::IsUnique() const
 // When the state of this mission changes, it may make changes to the player
 // information or show new UI panels. PlayerInfo::MissionCallback() will be
 // used as the callback for any UI panel that returns a value.
-bool Mission::Do(Trigger trigger, PlayerInfo &player, UI *ui)
+bool Mission::Do(Trigger trigger, PlayerInfo &player, UI *ui, const shared_ptr<Ship> &boardingShip)
 {
 	if(trigger == STOPOVER)
 	{
@@ -694,19 +778,31 @@ bool Mission::Do(Trigger trigger, PlayerInfo &player, UI *ui)
 				return false;
 			}
 		
+		visitedStopovers.insert(*it);
 		stopovers.erase(it);
 		if(!stopovers.empty())
 			return false;
 	}
+	// Don't update any conditions if this action exists and can't be completed.
+	auto it = actions.find(trigger);
+	if(it != actions.end() && !it->second.CanBeDone(player, boardingShip))
+		return false;
+	
 	if(trigger == ACCEPT)
 	{
 		++player.Conditions()[name + ": offered"];
 		++player.Conditions()[name + ": active"];
 	}
 	else if(trigger == DECLINE)
+	{
 		++player.Conditions()[name + ": offered"];
+		++player.Conditions()[name + ": declined"];
+	}
 	else if(trigger == FAIL)
+	{
 		--player.Conditions()[name + ": active"];
+		++player.Conditions()[name + ": failed"];
+	}
 	else if(trigger == COMPLETE)
 	{
 		--player.Conditions()[name + ": active"];
@@ -718,20 +814,13 @@ bool Mission::Do(Trigger trigger, PlayerInfo &player, UI *ui)
 	if(trigger == OFFER && location == JOB)
 		ui = nullptr;
 	
-	auto it = actions.find(trigger);
-	if(it == actions.end())
-	{
-		// If a mission has no "on offer" field, it is automatically accepted.
-		if(trigger == OFFER && location != JOB)
-			player.MissionCallback(Conversation::ACCEPT);
-		return true;
-	}
-	
-	if(!it->second.CanBeDone(player))
-		return false;
-	
-	// Perform any actions tied to this event.
-	it->second.Do(player, ui, destination ? destination->GetSystem() : nullptr);
+	// If this trigger has actions tied to it, perform them. Otherwise, check
+	// if this is a non-job mission that just got offered and if so,
+	// automatically accept it.
+	if(it != actions.end())
+		it->second.Do(player, ui, destination ? destination->GetSystem() : nullptr, boardingShip);
+	else if(trigger == OFFER && location != JOB)
+		player.MissionCallback(Conversation::ACCEPT);
 	
 	return true;
 }
@@ -751,20 +840,36 @@ const list<NPC> &Mission::NPCs() const
 // about it. This may affect the mission status or display a message.
 void Mission::Do(const ShipEvent &event, PlayerInfo &player, UI *ui)
 {
-	if(event.TargetGovernment()->IsPlayer() && !hasFailed
-			&& (event.Type() & ShipEvent::DESTROY))
+	if(event.TargetGovernment()->IsPlayer() && !hasFailed)
 	{
 		bool failed = false;
-		for(const auto &it : event.Target()->Cargo().MissionCargo())
-			failed |= (it.first == this);
-		for(const auto &it : event.Target()->Cargo().PassengerList())
-			failed |= (it.first == this);
+		string message = "Your ship '" + event.Target()->Name() + "' has been ";
+		if(event.Type() & ShipEvent::DESTROY)
+		{
+			// Destroyed ships carrying mission cargo result in failed missions.
+			// Mission cargo may have a quantity of zero (i.e. 0 mass).
+			for(const auto &it : event.Target()->Cargo().MissionCargo())
+				failed |= (it.first == this);
+			// If any mission passengers were present, this mission is failed.
+			for(const auto &it : event.Target()->Cargo().PassengerList())
+				failed |= (it.first == this && it.second);
+			if(failed)
+				message += "lost. ";
+		}
+		else if(event.Type() & ShipEvent::BOARD)
+		{
+			// Fail missions whose cargo is stolen by a boarding vessel.
+			for(const auto &it : event.Actor()->Cargo().MissionCargo())
+				failed |= (it.first == this);
+			if(failed)
+				message += "plundered. ";
+		}
 		
 		if(failed)
 		{
 			hasFailed = true;
 			if(isVisible)
-				Messages::Add("Ship lost. Mission failed: \"" + displayName + "\".");
+				Messages::Add(message + "Mission failed: \"" + displayName + "\".");
 		}
 	}
 	
@@ -773,11 +878,11 @@ void Mission::Do(const ShipEvent &event, PlayerInfo &player, UI *ui)
 	{
 		const System *system = event.Actor()->GetSystem();
 		// If this was a waypoint, clear it.
-		waypoints.erase(system);
+		if(waypoints.erase(system))
+			visitedWaypoints.insert(system);
 		
+		// Perform an "on enter" action for this system, if possible.
 		Enter(system, player, ui);
-		// Allow special "on enter" conditions that match any system.
-		Enter(nullptr, player, ui);
 	}
 	
 	for(NPC &npc : npcs)
@@ -798,7 +903,7 @@ const string &Mission::Identifier() const
 
 // "Instantiate" a mission by replacing randomly selected values and places
 // with a single choice, and then replacing any wildcard text as well.
-Mission Mission::Instantiate(const PlayerInfo &player) const
+Mission Mission::Instantiate(const PlayerInfo &player, const shared_ptr<Ship> &boardingShip) const
 {
 	Mission result;
 	// If anything goes wrong below, this mission should not be offered.
@@ -811,16 +916,18 @@ Mission Mission::Instantiate(const PlayerInfo &player) const
 	result.repeat = repeat;
 	result.name = name;
 	result.waypoints = waypoints;
-	// If one of the waypoints is the current system, it is already visited.
-	result.waypoints.erase(player.GetSystem());
 	// Handle waypoint systems that are chosen randomly.
+	const System * const source = player.GetSystem();
 	for(const LocationFilter &filter : waypointFilters)
 	{
-		const System *system = PickSystem(filter, player);
+		const System *system = filter.PickSystem(source);
 		if(!system)
 			return result;
 		result.waypoints.insert(system);
 	}
+	// If one of the waypoints is the current system, it is already visited.
+	if(result.waypoints.erase(source))
+		result.visitedWaypoints.insert(source);
 	
 	// Copy the stopover planet list, and populate the list based on the filters
 	// that were given.
@@ -835,7 +942,7 @@ Mission Mission::Instantiate(const PlayerInfo &player) const
 	}
 	for(const LocationFilter &filter : stopoverFilters)
 	{
-		const Planet *planet = PickPlanet(filter, player);
+		const Planet *planet = filter.PickPlanet(source, !clearance.empty());
 		if(!planet)
 			return result;
 		result.stopovers.insert(planet);
@@ -848,7 +955,7 @@ Mission Mission::Instantiate(const PlayerInfo &player) const
 	result.destination = destination;
 	if(!result.destination && !destinationFilter.IsEmpty())
 	{
-		result.destination = PickPlanet(destinationFilter, player);
+		result.destination = destinationFilter.PickPlanet(source, !clearance.empty());
 		if(!result.destination)
 			return result;
 	}
@@ -868,7 +975,7 @@ Mission Mission::Instantiate(const PlayerInfo &player) const
 	{
 		const Trade::Commodity *commodity = nullptr;
 		if(cargo == "random")
-			commodity = PickCommodity(*player.GetSystem(), *result.destination->GetSystem());
+			commodity = PickCommodity(*source, *result.destination->GetSystem());
 		else
 		{
 			for(const Trade::Commodity &option : GameData::Commodities())
@@ -916,7 +1023,7 @@ Mission Mission::Instantiate(const PlayerInfo &player) const
 	// Estimate how far the player will have to travel to visit all the waypoints
 	// and stopovers and then to land on the destination planet. Rather than a
 	// full traveling salesman path, just calculate a greedy approximation.
-	const System *source = player.GetSystem();
+	const System *path = source;
 	list<const System *> destinations;
 	for(const System *system : result.waypoints)
 		destinations.push_back(system);
@@ -927,18 +1034,18 @@ Mission Mission::Instantiate(const PlayerInfo &player) const
 	while(!destinations.empty())
 	{
 		// Find the closest destination to this location.
-		DistanceMap distance(source);
+		DistanceMap distance(path);
 		auto it = destinations.begin();
 		auto bestIt = it;
 		for(++it; it != destinations.end(); ++it)
 			if(distance.Days(*it) < distance.Days(*bestIt))
 				bestIt = it;
 		
-		source = *bestIt;
-		destinations.erase(bestIt);
+		path = *bestIt;
 		jumps += distance.Days(*bestIt);
+		destinations.erase(bestIt);
 	}
-	DistanceMap distance(source);
+	DistanceMap distance(path);
 	jumps += distance.Days(result.destination->GetSystem());
 	int payload = result.cargoSize + 10 * result.passengers;
 	
@@ -962,20 +1069,21 @@ Mission Mission::Instantiate(const PlayerInfo &player) const
 	subs["<fare>"] = (result.passengers == 1) ? "a passenger" : (subs["<bunks>"] + " passengers");
 	if(player.GetPlanet())
 		subs["<origin>"] = player.GetPlanet()->Name();
-	else if(player.BoardingShip())
-		subs["<origin>"] = player.BoardingShip()->Name();
+	else if(boardingShip)
+		subs["<origin>"] = boardingShip->Name();
 	subs["<planet>"] = result.destination ? result.destination->Name() : "";
 	subs["<system>"] = result.destination ? result.destination->GetSystem()->Name() : "";
 	subs["<destination>"] = subs["<planet>"] + " in the " + subs["<system>"] + " system";
 	subs["<date>"] = result.deadline.ToString();
 	subs["<day>"] = result.deadline.LongString();
+	// Stopover and waypoint substitutions: iterate by reference to the
+	// pointers so we can check when we're at the very last one in the set.
+	// Stopovers: "<name> in the <system name> system" with "," and "and".
 	if(!result.stopovers.empty())
 	{
 		string planets;
 		const Planet * const *last = &*--result.stopovers.end();
 		int count = 0;
-		// Iterate by reference to the pointers so we can check when we're at
-		// the very last one in the set.
 		for(const Planet * const &planet : result.stopovers)
 		{
 			if(count++)
@@ -984,17 +1092,33 @@ Mission Mission::Instantiate(const PlayerInfo &player) const
 		}
 		subs["<stopovers>"] = planets;
 	}
+	// Waypoints: "<system name>" with "," and "and".
+	if(!result.waypoints.empty())
+	{
+		string systems;
+		const System * const *last = &*--result.waypoints.end();
+		int count = 0;
+		for(const System * const &system : result.waypoints)
+		{
+			if(count++)
+				systems += (&system != last) ? ", " : (count > 2 ? ", and " : " and ");
+			systems += system->Name();
+		}
+		subs["<waypoints>"] = systems;
+	}
 	
 	// Instantiate the NPCs. This also fills in the "<npc>" substitution.
 	for(const NPC &npc : npcs)
-		result.npcs.push_back(npc.Instantiate(subs, player.GetSystem(), result.destination->GetSystem()));
+		result.npcs.push_back(npc.Instantiate(subs, source, result.destination->GetSystem()));
 	
 	// Instantiate the actions. The "complete" action is always first so that
 	// the "<payment>" substitution can be filled in.
 	for(const auto &it : actions)
-		result.actions[it.first] = it.second.Instantiate(subs, jumps, payload);
+		result.actions[it.first] = it.second.Instantiate(subs, source, jumps, payload);
 	for(const auto &it : onEnter)
-		result.onEnter[it.first] = it.second.Instantiate(subs, jumps, payload);
+		result.onEnter[it.first] = it.second.Instantiate(subs, source, jumps, payload);
+	for(const MissionAction &action : genericOnEnter)
+		result.genericOnEnter.emplace_back(action.Instantiate(subs, source, jumps, payload));
 	
 	// Perform substitution in the name and description.
 	result.displayName = Format::Replace(displayName, subs);
@@ -1010,48 +1134,44 @@ Mission Mission::Instantiate(const PlayerInfo &player) const
 
 
 
+// Perform an "on enter" MissionAction associated with the current system.
 void Mission::Enter(const System *system, PlayerInfo &player, UI *ui)
 {
-	auto eit = onEnter.find(system);
-	if(eit != onEnter.end() && !didEnter.count(eit->first))
+	const auto eit = onEnter.find(system);
+	if(eit != onEnter.end() && !didEnter.count(&eit->second) && eit->second.CanBeDone(player))
 	{
 		eit->second.Do(player, ui);
-		didEnter.insert(eit->first);
+		didEnter.insert(&eit->second);
 	}
+	// If no specific `on enter` was performed, try matching to a generic "on enter,"
+	// which may use a LocationFilter to govern which systems it can be performed in.
+	else
+		for(MissionAction &action : genericOnEnter)
+			if(!didEnter.count(&action) && action.CanBeDone(player))
+			{
+				action.Do(player, ui);
+				didEnter.insert(&action);
+				break;
+			}
 }
 
 
 
-const System *Mission::PickSystem(const LocationFilter &filter, const PlayerInfo &player) const
+// For legacy code, contraband definitions can be placed in two different
+// locations, so move that parsing out to a helper function.
+bool Mission::ParseContraband(const DataNode &node)
 {
-	// Find a planet that satisfies the filter.
-	vector<const System *> options;
-	for(const auto &it : GameData::Systems())
+	if(node.Token(0) == "illegal" && node.Size() == 2)
+		illegalCargoFine = node.Value(1);
+	else if(node.Token(0) == "illegal" && node.Size() == 3)
 	{
-		// Skip entries with incomplete data.
-		if(it.second.Name().empty())
-			continue;
-		if(filter.Matches(&it.second, player.GetSystem()))
-			options.push_back(&it.second);
+		illegalCargoFine = node.Value(1);
+		illegalCargoMessage = node.Token(2);
 	}
-	return options.empty() ? nullptr : options[Random::Int(options.size())];
-}
-
-
-
-const Planet *Mission::PickPlanet(const LocationFilter &filter, const PlayerInfo &player) const
-{
-	// Find a planet that satisfies the filter.
-	vector<const Planet *> options;
-	for(const auto &it : GameData::Planets())
-	{
-		// Skip entries with incomplete data.
-		if(it.second.Name().empty() || (clearance.empty() && !it.second.CanLand()))
-			continue;
-		if(it.second.IsWormhole() || !it.second.HasSpaceport())
-			continue;
-		if(filter.Matches(&it.second, player.GetSystem()))
-			options.push_back(&it.second);
-	}
-	return options.empty() ? nullptr : options[Random::Int(options.size())];
+	else if(node.Token(0) == "stealth")
+		failIfDiscovered = true;
+	else
+		return false;
+	
+	return true;
 }
