@@ -12,12 +12,14 @@ PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 
 #include "MissionAction.h"
 
+#include "CargoHold.h"
 #include "ConversationPanel.h"
 #include "DataNode.h"
 #include "DataWriter.h"
 #include "Dialog.h"
 #include "Format.h"
 #include "GameData.h"
+#include "GameEvent.h"
 #include "Messages.h"
 #include "Outfit.h"
 #include "PlayerInfo.h"
@@ -25,6 +27,7 @@ PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 #include "Ship.h"
 #include "UI.h"
 
+#include <cstdlib>
 #include <vector>
 
 using namespace std;
@@ -57,15 +60,17 @@ namespace {
 		
 		bool didCargo = false;
 		bool didShip = false;
-		int cargoCount = player.Cargo().Get(outfit);
+		// If not landed, transfers must be done into the flagship's CargoHold.
+		CargoHold &cargo = (player.GetPlanet() ? player.Cargo() : flagship->Cargo());
+		int cargoCount = cargo.Get(outfit);
 		if(count < 0 && cargoCount)
 		{
 			int moved = min(cargoCount, -count);
 			count += moved;
-			player.Cargo().Remove(outfit, moved);
+			cargo.Remove(outfit, moved);
 			didCargo = true;
 		}
-		while(flagship && count)
+		while(count)
 		{
 			int moved = (count > 0) ? 1 : -1;
 			if(flagship->Attributes().CanAdd(*outfit, moved))
@@ -80,12 +85,12 @@ namespace {
 		if(count > 0)
 		{
 			// Ignore cargo size limits.
-			int size = player.Cargo().Size();
-			player.Cargo().SetSize(-1);
-			player.Cargo().Add(outfit, count);
-			player.Cargo().SetSize(size);
+			int size = cargo.Size();
+			cargo.SetSize(-1);
+			cargo.Add(outfit, count);
+			cargo.SetSize(size);
 			didCargo = true;
-			if(count > 0 && ui)
+			if(ui)
 			{
 				string special = "The " + nameWas;
 				special += " put in your cargo hold because there is not enough space to install ";
@@ -102,6 +107,36 @@ namespace {
 			message += "flagship.";
 		Messages::Add(message);
 	}
+	
+	int CountInCargo(const Outfit *outfit, const PlayerInfo &player)
+	{
+		int available = 0;
+		// If landed, all cargo from available ships is pooled together.
+		if(player.GetPlanet())
+			available += player.Cargo().Get(outfit);
+		// Otherwise only count outfits in the cargo holds of in-system ships.
+		else
+		{
+			const System *here = player.GetSystem();
+			for(const auto &ship : player.Ships())
+			{
+				if(ship->IsDisabled() || ship->IsParked())
+					continue;
+				if(ship->GetSystem() == here || (ship->CanBeCarried()
+						&& !ship->GetSystem() && ship->GetParent()->GetSystem() == here))
+					available += ship->Cargo().Get(outfit);
+			}
+		}
+		return available;
+	}
+}
+
+
+
+// Construct and Load() at the same time.
+MissionAction::MissionAction(const DataNode &node, const string &missionName)
+{
+	Load(node, missionName);
 }
 
 
@@ -118,24 +153,34 @@ void MissionAction::Load(const DataNode &node, const string &missionName)
 		const string &key = child.Token(0);
 		bool hasValue = (child.Size() >= 2);
 		
-		if(key == "log" || key == "dialog")
+		if(key == "log")
 		{
-			bool isSpecial = (key == "log" && child.Size() >= 3);
-			string &text = (key == "dialog" ? dialogText :
-				isSpecial ? specialLogText[child.Token(1)][child.Token(2)] : logText);
-			for(int i = isSpecial ? 3 : 1; i < child.Size(); ++i)
+			bool isSpecial = (child.Size() >= 3);
+			string &text = (isSpecial ?
+				specialLogText[child.Token(1)][child.Token(2)] : logText);
+			Dialog::ParseTextNode(child, isSpecial ? 3 : 1, text);
+		}
+		else if(key == "dialog")
+		{
+			// Dialog text may be supplied from a stock named phrase, a
+			// private unnamed phrase, or directly specified.
+			if(hasValue && child.Token(1) == "phrase")
 			{
-				if(!text.empty())
-					text += "\n\t";
-				text += child.Token(i);
+				if(!child.HasChildren() && child.Size() == 3)
+					stockDialogPhrase = GameData::Phrases().Get(child.Token(2));
+				else
+					child.PrintTrace("Skipping unsupported dialog phrase syntax:");
 			}
-			for(const DataNode &grand : child)
-				for(int i = 0; i < grand.Size(); ++i)
-				{
-					if(!text.empty())
-						text += "\n\t";
-					text += grand.Token(i);
-				}
+			else if(!hasValue && child.HasChildren() && (*child.begin()).Token(0) == "phrase")
+			{
+				const DataNode &firstGrand = (*child.begin());
+				if(firstGrand.Size() == 1 && firstGrand.HasChildren())
+					dialogPhrase.Load(firstGrand);
+				else
+					firstGrand.PrintTrace("Skipping unsupported dialog phrase syntax:");
+			}
+			else
+				Dialog::ParseTextNode(child, 1, dialogText);
 		}
 		else if(key == "conversation" && child.HasChildren())
 			conversation.Load(child);
@@ -144,10 +189,23 @@ void MissionAction::Load(const DataNode &node, const string &missionName)
 		else if(key == "outfit" && hasValue)
 		{
 			int count = (child.Size() < 3 ? 1 : static_cast<int>(child.Value(2)));
-			gifts[GameData::Outfits().Get(child.Token(1))] = count;
+			if(count)
+				gifts[GameData::Outfits().Get(child.Token(1))] = count;
+			else
+			{
+				// outfit <outfit> 0 means the player must have this outfit.
+				child.PrintTrace("Warning: deprecated use of \"outfit\" with count of 0. Use \"require <outfit>\" instead:");
+				requiredOutfits[GameData::Outfits().Get(child.Token(1))] = 1;
+			}
 		}
 		else if(key == "require" && hasValue)
-			gifts[GameData::Outfits().Get(child.Token(1))] = 0;
+		{
+			int count = (child.Size() < 3 ? 1 : static_cast<int>(child.Value(2)));
+			if(count >= 0)
+				requiredOutfits[GameData::Outfits().Get(child.Token(1))] = count;
+			else
+				child.PrintTrace("Skipping invalid \"require\" amount:");
+		}
 		else if(key == "payment")
 		{
 			if(child.Size() == 1)
@@ -163,10 +221,22 @@ void MissionAction::Load(const DataNode &node, const string &missionName)
 			int maxDays = (child.Size() >= 4 ? child.Value(3) : minDays);
 			if(maxDays < minDays)
 				swap(minDays, maxDays);
-			events[child.Token(1)] = make_pair(minDays, maxDays);
+			events[GameData::Events().Get(child.Token(1))] = make_pair(minDays, maxDays);
 		}
 		else if(key == "fail")
-			fail.insert(child.Size() >= 2 ? child.Token(1) : missionName);
+		{
+			string toFail = child.Size() >= 2 ? child.Token(1) : missionName;
+			fail.insert(toFail);
+			// Create a GameData reference to this mission name.
+			GameData::Missions().Get(toFail);
+		}
+		else if(key == "system")
+		{
+			if(system.empty() && child.HasChildren())
+				systemFilter.Load(child);
+			else
+				child.PrintTrace("Unsupported use of \"system\" LocationFilter:");
+		}
 		else
 			conditions.Add(child);
 	}
@@ -184,6 +254,12 @@ void MissionAction::Save(DataWriter &out) const
 		out.Write("on", trigger, system);
 	out.BeginChild();
 	{
+		if(!systemFilter.IsEmpty())
+		{
+			out.Write("system");
+			// LocationFilter indentation is handled by its Save method.
+			systemFilter.Save(out);
+		}
 		if(!logText.empty())
 		{
 			out.Write("log");
@@ -223,14 +299,16 @@ void MissionAction::Save(DataWriter &out) const
 		
 		for(const auto &it : gifts)
 			out.Write("outfit", it.first->Name(), it.second);
+		for(const auto &it : requiredOutfits)
+			out.Write("require", it.first->Name(), it.second);
 		if(payment)
 			out.Write("payment", payment);
 		for(const auto &it : events)
 		{
 			if(it.second.first == it.second.second)
-				out.Write("event", it.first, it.second.first);
+				out.Write("event", it.first->Name(), it.second.first);
 			else
-				out.Write("event", it.first, it.second.first, it.second.second);
+				out.Write("event", it.first->Name(), it.second.first, it.second.second);
 		}
 		for(const auto &name : fail)
 			out.Write("fail", name);
@@ -251,7 +329,7 @@ int MissionAction::Payment() const
 
 // Check if this action can be completed right now. It cannot be completed
 // if it takes away money or outfits that the player does not have.
-bool MissionAction::CanBeDone(const PlayerInfo &player) const
+bool MissionAction::CanBeDone(const PlayerInfo &player, const shared_ptr<Ship> &boardingShip) const
 {
 	if(player.Accounts().Credits() < -payment)
 		return false;
@@ -259,34 +337,77 @@ bool MissionAction::CanBeDone(const PlayerInfo &player) const
 	const Ship *flagship = player.Flagship();
 	for(const auto &it : gifts)
 	{
+		// If this outfit is being given, the player doesn't need to have it.
 		if(it.second > 0)
 			continue;
 		
-		// The outfit can be taken from the player's cargo or from the flagship.
-		int available = player.Cargo().Get(it.first);
-		for(const auto &ship : player.Ships())
-			available += ship->Cargo().Get(it.first);
-		if(flagship)
-			available += flagship->OutfitCount(it.first);
+		// Outfits may always be taken from the flagship. If landed, they may also be taken from
+		// the collective cargohold of any in-system, non-disabled escorts (player.Cargo()). If
+		// boarding, consider only the flagship's cargo hold. If in-flight, show mission status
+		// by checking the cargo holds of ships that would contribute to player.Cargo if landed.
+		int available = flagship ? flagship->OutfitCount(it.first) : 0;
+		available += boardingShip ? flagship->Cargo().Get(it.first)
+				: CountInCargo(it.first, player);
 		
-		// If the gift "count" is 0, that means to check that the player has at
-		// least one of these items.
-		if(available < -it.second + !it.second)
+		if(available < -it.second)
 			return false;
 	}
+	
+	for(const auto &it : requiredOutfits)
+	{
+		int available = 0;
+		// Requiring the player to have 0 of this outfit means all ships and all cargo holds
+		// must be checked, even if the ship is disabled, parked, or out-of-system.
+		bool checkAll = !it.second;
+		if(checkAll)
+		{
+			for(const auto &ship : player.Ships())
+				if(!ship->IsDestroyed())
+				{
+					available += ship->Cargo().Get(it.first);
+					available += ship->OutfitCount(it.first);
+				}
+		}
+		else
+		{
+			// Required outfits must be present on able ships in the
+			// player's location (or the respective cargo hold).
+			available += flagship ? flagship->OutfitCount(it.first) : 0;
+			available += boardingShip ? flagship->Cargo().Get(it.first)
+					: CountInCargo(it.first, player);
+		}
+		
+		if(available < it.second)
+			return false;
+		
+		// If the required count is 0, the player must not have any of the outfit.
+		if(checkAll && available)
+			return false;
+	}
+	
+	// An `on enter` MissionAction may have defined a LocationFilter that
+	// specifies the systems in which it can occur.
+	if(!systemFilter.IsEmpty() && !systemFilter.Matches(player.GetSystem()))
+		return false;
 	return true;
 }
 
 
 
-void MissionAction::Do(PlayerInfo &player, UI *ui, const System *destination) const
+void MissionAction::Do(PlayerInfo &player, UI *ui, const System *destination, const shared_ptr<Ship> &ship) const
 {
 	bool isOffer = (trigger == "offer");
 	if(!conversation.IsEmpty() && ui)
 	{
-		ConversationPanel *panel = new ConversationPanel(player, conversation, destination);
+		// Conversations offered while boarding or assisting reference a ship,
+		// which may be destroyed depending on the player's choices.
+		ConversationPanel *panel = new ConversationPanel(player, conversation, destination, ship);
 		if(isOffer)
 			panel->SetCallback(&player, &PlayerInfo::MissionCallback);
+		// Use a basic callback to handle forced departure outside of `on offer`
+		// conversations.
+		else
+			panel->SetCallback(&player, &PlayerInfo::BasicCallback);
 		ui->Push(panel);
 	}
 	else if(!dialogText.empty() && ui)
@@ -325,7 +446,7 @@ void MissionAction::Do(PlayerInfo &player, UI *ui, const System *destination) co
 		player.Accounts().AddCredits(payment);
 	
 	for(const auto &it : events)
-		player.AddEvent(*GameData::Events().Get(it.first), player.GetDate() + it.second.first);
+		player.AddEvent(*it.first, player.GetDate() + it.second.first);
 	
 	if(!fail.empty())
 	{
@@ -345,11 +466,13 @@ void MissionAction::Do(PlayerInfo &player, UI *ui, const System *destination) co
 
 
 
-MissionAction MissionAction::Instantiate(map<string, string> &subs, int jumps, int payload) const
+MissionAction MissionAction::Instantiate(map<string, string> &subs, const System *origin, int jumps, int payload) const
 {
 	MissionAction result;
 	result.trigger = trigger;
 	result.system = system;
+	// Convert any "distance" specifiers into "near <system>" specifiers.
+	result.systemFilter = systemFilter.SetOrigin(origin);
 	
 	for(const auto &it : events)
 	{
@@ -364,7 +487,7 @@ MissionAction MissionAction::Instantiate(map<string, string> &subs, int jumps, i
 	// Fill in the payment amount if this is the "complete" action.
 	string previousPayment = subs["<payment>"];
 	if(result.payment)
-		subs["<payment>"] = Format::Number(result.payment)
+		subs["<payment>"] = Format::Credits(abs(result.payment))
 			+ (result.payment == 1 ? " credit" : " credits");
 	
 	if(!logText.empty())
@@ -373,6 +496,10 @@ MissionAction MissionAction::Instantiate(map<string, string> &subs, int jumps, i
 		for(const auto &eit : it.second)
 			result.specialLogText[it.first][eit.first] = Format::Replace(eit.second, subs);
 	
+	// Create any associated dialog text from phrases, or use the directly specified text.
+	string dialogText = stockDialogPhrase ? stockDialogPhrase->Get()
+		: (!dialogPhrase.Name().empty() ? dialogPhrase.Get()
+		: this->dialogText);
 	if(!dialogText.empty())
 		result.dialogText = Format::Replace(dialogText, subs);
 	
