@@ -37,12 +37,33 @@ PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 #include "System.h"
 #include "UI.h"
 
+#include <algorithm>
 #include <sstream>
 
 using namespace std;
 
 namespace {
-	static const int SIDE_WIDTH = 280;
+	const int SIDE_WIDTH = 280;
+	
+	// Check if the mission involves the given system,
+	bool Involves(const Mission &mission, const System *system)
+	{
+		if(!system)
+			return false;
+		
+		if(mission.Destination()->IsInSystem(system))
+			return true;
+		
+		for(const System *waypoint : mission.Waypoints())
+			if(waypoint == system)
+				return true;
+		
+		for(const Planet *stopover : mission.Stopovers())
+			if(stopover->IsInSystem(system))
+				return true;
+		
+		return false;
+	}
 }
 
 
@@ -60,7 +81,7 @@ MissionPanel::MissionPanel(PlayerInfo &player)
 	wrap.SetWrapWidth(380);
 	wrap.SetFont(FontSet::Get(14));
 	wrap.SetAlignment(WrappedText::JUSTIFIED);
-
+	
 	// Select the first available or accepted mission in the currently selected
 	// system, or along the travel plan.
 	if(!FindMissionForSystem(selectedSystem) && player.HasTravelPlan())
@@ -70,16 +91,15 @@ MissionPanel::MissionPanel(PlayerInfo &player)
 			if(FindMissionForSystem(*it))
 				break;
 	}
-
+	
 	// Auto select the destination system for the current mission.
 	if(availableIt != available.end())
 		selectedSystem = availableIt->Destination()->GetSystem();
 	else if(acceptedIt != accepted.end())
 		selectedSystem = acceptedIt->Destination()->GetSystem();
-
-	// Center the system slightly above the center of the screen because the
-	// lower panel is taking up more space than the upper one.
-	center = Point(0., -80.) - selectedSystem->Position();
+	
+	// Center on the selected system.
+	CenterOnSystem(selectedSystem, true);
 }
 
 
@@ -119,6 +139,7 @@ MissionPanel::MissionPanel(const MapPanel &panel)
 
 void MissionPanel::Step()
 {
+	MapPanel::Step();
 	DoHelp("jobs");
 }
 
@@ -128,7 +149,7 @@ void MissionPanel::Draw()
 {
 	MapPanel::Draw();
 	
-	Color routeColor(.2, .1, 0., 0.);
+	Color routeColor(.2f, .1f, 0.f, 0.f);
 	const System *system = selectedSystem;
 	while(distance.Days(system) > 0)
 	{
@@ -140,7 +161,7 @@ void MissionPanel::Draw()
 		from -= unit;
 		to += unit;
 		
-		LineShader::Draw(from, to, 5., routeColor);
+		LineShader::Draw(from, to, 5.f, routeColor);
 		
 		system = next;
 	}
@@ -177,7 +198,7 @@ void MissionPanel::Draw()
 
 
 // Only override the ones you need; the default action is to return false.
-bool MissionPanel::KeyDown(SDL_Keycode key, Uint16 mod, const Command &command)
+bool MissionPanel::KeyDown(SDL_Keycode key, Uint16 mod, const Command &command, bool isNewPress)
 {
 	if(key == 'a' && CanAccept())
 	{
@@ -239,14 +260,14 @@ bool MissionPanel::KeyDown(SDL_Keycode key, Uint16 mod, const Command &command)
 		}
 	}
 	else
-		return MapPanel::KeyDown(key, mod, command);
+		return MapPanel::KeyDown(key, mod, command, isNewPress);
 	
 	if(availableIt != available.end())
 		selectedSystem = availableIt->Destination()->GetSystem();
 	else if(acceptedIt != accepted.end())
 		selectedSystem = acceptedIt->Destination()->GetSystem();
 	if(selectedSystem)
-		center = Point(0., -80.) - selectedSystem->Position();
+		CenterOnSystem(selectedSystem);
 	
 	return true;
 }
@@ -271,7 +292,7 @@ bool MissionPanel::Click(int x, int y, int clicks)
 			acceptedIt = accepted.end();
 			dragSide = -1;
 			selectedSystem = availableIt->Destination()->GetSystem();
-			center = Point(0., -80.) - selectedSystem->Position();
+			CenterOnSystem(selectedSystem);
 			return true;
 		}
 	}
@@ -289,7 +310,7 @@ bool MissionPanel::Click(int x, int y, int clicks)
 			availableIt = available.end();
 			dragSide = 1;
 			selectedSystem = acceptedIt->Destination()->GetSystem();
-			center = Point(0., -80.) - selectedSystem->Position();
+			CenterOnSystem(selectedSystem);
 			return true;
 		}
 	}
@@ -344,9 +365,9 @@ bool MissionPanel::Click(int x, int y, int clicks)
 			if(acceptedIt != accepted.end() && !acceptedIt->IsVisible())
 				continue;
 			
-			if(availableIt != available.end() && availableIt->Destination()->IsInSystem(system))
+			if(availableIt != available.end() && Involves(*availableIt, system))
 				break;
-			if(acceptedIt != accepted.end() && acceptedIt->Destination()->IsInSystem(system))
+			if(acceptedIt != accepted.end() && Involves(*acceptedIt, system))
 				break;
 		}
 		// Make sure invisible missions are never selected, even if there were
@@ -382,6 +403,7 @@ bool MissionPanel::Drag(double dx, double dy)
 
 
 
+// Check to see if the mouse is over either of the mission lists.
 bool MissionPanel::Hover(int x, int y)
 {
 	dragSide = 0;
@@ -396,7 +418,7 @@ bool MissionPanel::Hover(int x, int y)
 		if(static_cast<int>(index) < AcceptedVisible())
 			dragSide = 1;
 	}
-	return true;
+	return dragSide ? true : MapPanel::Hover(x, y);
 }
 
 
@@ -416,24 +438,25 @@ void MissionPanel::DrawKey() const
 	const Sprite *back = SpriteSet::Get("ui/mission key");
 	SpriteShader::Draw(back, Screen::BottomLeft() + .5 * Point(back->Width(), -back->Height()));
 	
-	Color bright(.6, .6);
-	Color dim(.3, .3);
 	const Font &font = FontSet::Get(14);
 	Point angle = Point(1., 1.).Unit();
 	
-	Point pos(Screen::Left() + 10., Screen::Bottom() - 5. * 20. + 5.);
+	const int ROWS = 5;
+	Point pos(Screen::Left() + 10., Screen::Bottom() - ROWS * 20. + 5.);
 	Point pointerOff(5., 5.);
 	Point textOff(8., -.5 * font.Height());
 
 	const Set<Color> &colors = GameData::Colors();
-	const Color COLOR[5] = {
+	const Color &bright = *colors.Get("bright");
+	const Color &dim = *colors.Get("dim");
+	const Color COLOR[ROWS] = {
 		*colors.Get("available job"),
 		*colors.Get("unavailable job"),
 		*colors.Get("active mission"),
 		*colors.Get("blocked mission"),
 		*colors.Get("waypoint")
 	};
-	static const string LABEL[5] = {
+	static const string LABEL[ROWS] = {
 		"Available job; can accept",
 		"Too little space to accept",
 		"Active job; go here to complete",
@@ -446,9 +469,9 @@ void MissionPanel::DrawKey() const
 	if(acceptedIt != accepted.end() && acceptedIt->Destination())
 		selected = 2 + !IsSatisfied(*acceptedIt);
 	
-	for(int i = 0; i < 5; ++i)
+	for(int i = 0; i < ROWS; ++i)
 	{
-		PointerShader::Draw(pos + pointerOff, angle, 10., 18., 0., COLOR[i]);
+		PointerShader::Draw(pos + pointerOff, angle, 10.f, 18.f, 0.f, COLOR[i]);
 		font.Draw(LABEL[i], pos + textOff, i == selected ? bright : dim);
 		pos.Y() += 20.;
 	}
@@ -456,10 +479,11 @@ void MissionPanel::DrawKey() const
 
 
 
+// Fill in the top-middle header bar that names the selected system, and indicates its distance.
 void MissionPanel::DrawSelectedSystem() const
 {
 	const Sprite *sprite = SpriteSet::Get("ui/selected system");
-	SpriteShader::Draw(sprite, Point(0., Screen::Top() + .5 * sprite->Height()));
+	SpriteShader::Draw(sprite, Point(0., Screen::Top() + .5f * sprite->Height()));
 	
 	string text;
 	if(!selectedSystem)
@@ -489,26 +513,41 @@ void MissionPanel::DrawSelectedSystem() const
 
 
 
+// Highlight the systems associated with the given mission (i.e. destination and
+// waypoints) by drawing colored rings around them.
 void MissionPanel::DrawMissionSystem(const Mission &mission, const Color &color) const
 {
-	const Color &waypointColor = *GameData::Colors().Get("waypoint back");
+	const Color &waypoint = *GameData::Colors().Get("waypoint back");
+	const Color &visited = *GameData::Colors().Get("faint");
+	const float MISSION_OUTER = 22.f;
+	const float MISSION_INNER = 20.5f;
 	
-	Point pos = Zoom() * (mission.Destination()->GetSystem()->Position() + center);
-	RingShader::Draw(pos, 22., 20.5, color);
+	double zoom = Zoom();
+	// Draw a colored ring around the destination system.
+	Point pos = zoom * (mission.Destination()->GetSystem()->Position() + center);
+	RingShader::Draw(pos, MISSION_OUTER, MISSION_INNER, color);
+	
+	// Draw bright rings around systems that still need to be visited.
 	for(const System *system : mission.Waypoints())
-		RingShader::Draw(Zoom() * (system->Position() + center), 22., 20.5, waypointColor);
+		RingShader::Draw(zoom * (system->Position() + center), MISSION_OUTER, MISSION_INNER, waypoint);
 	for(const Planet *planet : mission.Stopovers())
-		RingShader::Draw(Zoom() * (planet->GetSystem()->Position() + center), 22., 20.5, waypointColor);
+		RingShader::Draw(zoom * (planet->GetSystem()->Position() + center), MISSION_OUTER, MISSION_INNER, waypoint);
+	
+	// Draw faint rings around systems already visited for this mission.
+	for(const System *system : mission.VisitedWaypoints())
+		RingShader::Draw(zoom * (system->Position() + center), MISSION_OUTER, MISSION_INNER, visited);
+	for(const Planet *planet : mission.VisitedStopovers())
+		RingShader::Draw(zoom * (planet->GetSystem()->Position() + center), MISSION_OUTER, MISSION_INNER, visited);
 }
 
 
 
+// Draw the background for the lists of available and accepted missions (based on pos).
 Point MissionPanel::DrawPanel(Point pos, const string &label, int entries) const
 {
-	const Font &font = FontSet::Get(14);
-	Color back(.125, 1.);
-	Color unselected = *GameData::Colors().Get("medium");
-	Color selected = *GameData::Colors().Get("bright");
+	const Color &back = *GameData::Colors().Get("map side panel background");
+	const Color &unselected = *GameData::Colors().Get("medium");
+	const Color &selected = *GameData::Colors().Get("bright");
 	
 	// Draw the panel.
 	Point size(SIDE_WIDTH, 20 * entries + 40);
@@ -533,6 +572,7 @@ Point MissionPanel::DrawPanel(Point pos, const string &label, int entries) const
 		edgePos.Y() -= dy;
 	}
 	
+	const Font &font = FontSet::Get(14);
 	pos += Point(10., 10. + (20. - font.Height()) * .5);
 	font.Draw(label, pos, selected);
 	FillShader::Fill(
@@ -549,10 +589,10 @@ Point MissionPanel::DrawPanel(Point pos, const string &label, int entries) const
 Point MissionPanel::DrawList(const list<Mission> &list, Point pos) const
 {
 	const Font &font = FontSet::Get(14);
-	Color highlight = *GameData::Colors().Get("faint");
-	Color unselected = *GameData::Colors().Get("medium");
-	Color selected = *GameData::Colors().Get("bright");
-	Color dim = *GameData::Colors().Get("dim");
+	const Color &highlight = *GameData::Colors().Get("faint");
+	const Color &unselected = *GameData::Colors().Get("medium");
+	const Color &selected = *GameData::Colors().Get("bright");
+	const Color &dim = *GameData::Colors().Get("dim");
 	
 	for(auto it = list.begin(); it != list.end(); ++it)
 	{
@@ -590,7 +630,7 @@ void MissionPanel::DrawMissionInfo()
 		info.SetCondition("can abort");
 	
 	info.SetString("cargo free", to_string(player.Cargo().Free()) + " tons");
-	info.SetString("bunks free", to_string(player.Cargo().Bunks()) + " bunks");
+	info.SetString("bunks free", to_string(player.Cargo().BunksFree()) + " bunks");
 	
 	info.SetString("today", player.GetDate().ToString());
 	
@@ -627,7 +667,7 @@ void MissionPanel::Accept()
 		cargoToSell = toAccept.CargoSize() - player.Cargo().Free();
 	int crewToFire = 0;
 	if(toAccept.Passengers())
-		crewToFire = toAccept.Passengers() - player.Cargo().Bunks();
+		crewToFire = toAccept.Passengers() - player.Cargo().BunksFree();
 	if(cargoToSell > 0 || crewToFire > 0)
 	{
 		ostringstream out;
@@ -671,7 +711,7 @@ void MissionPanel::MakeSpaceAndAccept()
 {
 	const Mission &toAccept = *availableIt;
 	int cargoToSell = toAccept.CargoSize() - player.Cargo().Free();
-	int crewToFire = toAccept.Passengers() - player.Cargo().Bunks();
+	int crewToFire = toAccept.Passengers() - player.Cargo().BunksFree();
 	
 	if(crewToFire > 0)
 		player.Flagship()->AddCrew(-crewToFire);
