@@ -12,13 +12,13 @@ PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 
 #include "OutfitInfoDisplay.h"
 
-#include "Color.h"
 #include "Depreciation.h"
 #include "Format.h"
 #include "Outfit.h"
 #include "PlayerInfo.h"
 
 #include <algorithm>
+#include <cmath>
 #include <map>
 #include <set>
 #include <sstream>
@@ -26,25 +26,35 @@ PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 using namespace std;
 
 namespace {
-	static const map<string, double> SCALE = {
+	const map<string, double> SCALE = {
 		{"active cooling", 60.},
 		{"afterburner energy", 60.},
 		{"afterburner fuel", 60.},
 		{"afterburner heat", 60.},
+		{"cloak", 60.},
 		{"cloaking energy", 60.},
 		{"cloaking fuel", 60.},
+		{"cloaking heat", 60.},
 		{"cooling", 60.},
 		{"cooling energy", 60.},
 		{"energy consumption", 60.},
 		{"energy generation", 60.},
+		{"fuel consumption", 60.},
+		{"fuel energy", 60.},
+		{"fuel generation", 60.},
+		{"fuel heat", 60.},
 		{"heat generation", 60.},
+		{"heat dissipation", 60.},
 		{"hull repair rate", 60.},
 		{"hull energy", 60.},
+		{"hull fuel", 60.},
 		{"hull heat", 60.},
+		{"jump speed", 60.},
 		{"reverse thrusting energy", 60.},
 		{"reverse thrusting heat", 60.},
 		{"shield generation", 60.},
 		{"shield energy", 60.},
+		{"shield fuel", 60.},
 		{"shield heat", 60.},
 		{"solar collection", 60.},
 		{"thrusting energy", 60.},
@@ -62,7 +72,7 @@ namespace {
 		{"slowing resistance", 60. * 100.}
 	};
 	
-	static const map<string, string> BOOLEAN_ATTRIBUTES = {
+	const map<string, string> BOOLEAN_ATTRIBUTES = {
 		{"unplunderable", "This outfit cannot be plundered."},
 		{"installable", "This is not an installable item."},
 		{"hyperdrive", "Allows you to make hyperjumps."},
@@ -124,7 +134,7 @@ void OutfitInfoDisplay::UpdateRequirements(const Outfit &outfit, const PlayerInf
 		out << "cost (" << (100 * buyValue) / cost << "%):";
 		requirementLabels.push_back(out.str());
 	}
-	requirementValues.push_back(Format::Number(buyValue));
+	requirementValues.push_back(Format::Credits(buyValue));
 	requirementsHeight += 20;
 	
 	if(canSell && sellValue != buyValue)
@@ -137,28 +147,44 @@ void OutfitInfoDisplay::UpdateRequirements(const Outfit &outfit, const PlayerInf
 			out << "sells for (" << (100 * sellValue) / cost << "%):";
 			requirementLabels.push_back(out.str());
 		}
-		requirementValues.push_back(Format::Number(sellValue));
+		requirementValues.push_back(Format::Credits(sellValue));
 		requirementsHeight += 20;
 	}
 	
+	if(outfit.Mass())
+	{
+		requirementLabels.emplace_back("mass:");
+		requirementValues.emplace_back(Format::Number(outfit.Mass()));
+		requirementsHeight += 20;
+	}
+	
+	bool hasContent = true;
 	static const vector<string> NAMES = {
+		"", "",
 		"outfit space needed:", "outfit space",
 		"weapon capacity needed:", "weapon capacity",
 		"engine capacity needed:", "engine capacity",
+		"", "",
 		"gun ports needed:", "gun ports",
 		"turret mounts needed:", "turret mounts"
 	};
 	for(unsigned i = 0; i + 1 < NAMES.size(); i += 2)
-		if(outfit.Get(NAMES[i + 1]))
+	{
+		if(NAMES[i].empty() && hasContent)
 		{
-			requirementLabels.push_back(string());
-			requirementValues.push_back(string());
+			requirementLabels.emplace_back();
+			requirementValues.emplace_back();
 			requirementsHeight += 10;
-		
+			hasContent = false;
+		}
+		else if(outfit.Get(NAMES[i + 1]))
+		{
 			requirementLabels.push_back(NAMES[i]);
 			requirementValues.push_back(Format::Number(-outfit.Get(NAMES[i + 1])));
 			requirementsHeight += 20;
+			hasContent = true;
 		}
+	}
 }
 
 
@@ -169,140 +195,122 @@ void OutfitInfoDisplay::UpdateAttributes(const Outfit &outfit)
 	attributeValues.clear();
 	attributesHeight = 20;
 	
-	map<string, map<string, int>> listing;
-	for(const auto &it : outfit.Attributes())
+	bool hasNormalAttributes = false;
+	for(const pair<const char *, double> &it : outfit.Attributes())
 	{
-		if(it.first == "outfit space"
-				|| it.first == "weapon capacity" || it.first == "engine capacity"
-				|| it.first == "gun ports" || it.first == "turret mounts")
+		static const set<string> SKIP = {
+			"outfit space", "weapon capacity", "engine capacity", "gun ports", "turret mounts"
+		};
+		if(SKIP.count(it.first))
 			continue;
 		
-		string value;
 		auto sit = SCALE.find(it.first);
 		double scale = (sit == SCALE.end() ? 1. : sit->second);
 		
 		auto bit = BOOLEAN_ATTRIBUTES.find(it.first);
 		if(bit != BOOLEAN_ATTRIBUTES.end()) 
 		{
-			attributeLabels.push_back(bit->second);
-			attributeValues.push_back(" ");
+			attributeLabels.emplace_back(bit->second);
+			attributeValues.emplace_back(" ");
 			attributesHeight += 20;
 		}
 		else
 		{
-			attributeLabels.push_back(it.first + ':');
-			attributeValues.push_back(Format::Number(it.second * scale));
+			attributeLabels.emplace_back(static_cast<string>(it.first) + ":");
+			attributeValues.emplace_back(Format::Number(it.second * scale));
 			attributesHeight += 20;
 		}
+		hasNormalAttributes = true;
 	}
 	
 	if(!outfit.IsWeapon())
 		return;
 	
-	attributeLabels.push_back(string());
-	attributeValues.push_back(string());
-	attributesHeight += 10;
+	// Insert padding if any normal attributes were listed above.
+	if(hasNormalAttributes)
+	{
+		attributeLabels.emplace_back();
+		attributeValues.emplace_back();
+		attributesHeight += 10;
+	}
 	
 	if(outfit.Ammo())
 	{
-		attributeLabels.push_back("ammo:");
-		attributeValues.push_back(outfit.Ammo()->Name());
+		attributeLabels.emplace_back("ammo:");
+		attributeValues.emplace_back(outfit.Ammo()->Name());
 		attributesHeight += 20;
 	}
 	
-	attributeLabels.push_back("range:");
-	attributeValues.push_back(Format::Number(outfit.Range()));
+	attributeLabels.emplace_back("range:");
+	attributeValues.emplace_back(Format::Number(outfit.Range()));
 	attributesHeight += 20;
 	
-	if(outfit.ShieldDamage() && outfit.Reload())
+	static const vector<string> VALUE_NAMES = {
+		"shield damage",
+		"hull damage",
+		"fuel damage",
+		"heat damage",
+		"ion damage",
+		"slowing damage",
+		"disruption damage",
+		"firing energy",
+		"firing heat",
+		"firing fuel"
+	};
+	
+	vector<double> values = {
+		outfit.ShieldDamage(),
+		outfit.HullDamage(),
+		outfit.FuelDamage(),
+		outfit.HeatDamage(),
+		outfit.IonDamage() * 100.,
+		outfit.SlowingDamage() * 100.,
+		outfit.DisruptionDamage() * 100.,
+		outfit.FiringEnergy(),
+		outfit.FiringHeat(),
+		outfit.FiringFuel()
+	};
+	
+	// Add any per-second values to the table.
+	double reload = outfit.Reload();
+	if(reload)
 	{
-		attributeLabels.push_back("shield damage / second:");
-		attributeValues.push_back(Format::Number(60. * outfit.ShieldDamage() / outfit.Reload()));
-		attributesHeight += 20;
+		static const string PER_SECOND = " / second:";
+		for(unsigned i = 0; i < values.size(); ++i)
+			if(values[i])
+			{
+				attributeLabels.emplace_back(VALUE_NAMES[i] + PER_SECOND);
+				attributeValues.emplace_back(Format::Number(60. * values[i] / reload));
+				attributesHeight += 20;
+			}
 	}
 	
-	if(outfit.HullDamage() && outfit.Reload())
-	{
-		attributeLabels.push_back("hull damage / second:");
-		attributeValues.push_back(Format::Number(60. * outfit.HullDamage() / outfit.Reload()));
-		attributesHeight += 20;
-	}
-	
-	if(outfit.HeatDamage() && outfit.Reload())
-	{
-		attributeLabels.push_back("heat damage / second:");
-		attributeValues.push_back(Format::Number(60. * outfit.HeatDamage() / outfit.Reload()));
-		attributesHeight += 20;
-	}
-	
-	if(outfit.IonDamage() && outfit.Reload())
-	{
-		attributeLabels.push_back("ion damage / second:");
-		attributeValues.push_back(Format::Number(6000. * outfit.IonDamage() / outfit.Reload()));
-		attributesHeight += 20;
-	}
-	
-	if(outfit.SlowingDamage() && outfit.Reload())
-	{
-		attributeLabels.push_back("slowing damage / second:");
-		attributeValues.push_back(Format::Number(6000. * outfit.SlowingDamage() / outfit.Reload()));
-		attributesHeight += 20;
-	}
-	
-	if(outfit.DisruptionDamage() && outfit.Reload())
-	{
-		attributeLabels.push_back("disruption damage / second:");
-		attributeValues.push_back(Format::Number(6000. * outfit.DisruptionDamage() / outfit.Reload()));
-		attributesHeight += 20;
-	}
-	
-	if(outfit.FiringEnergy() && outfit.Reload())
-	{
-		attributeLabels.push_back("firing energy / second:");
-		attributeValues.push_back(Format::Number(60. * outfit.FiringEnergy() / outfit.Reload()));
-		attributesHeight += 20;
-	}
-	
-	if(outfit.FiringHeat() && outfit.Reload())
-	{
-		attributeLabels.push_back("firing heat / second:");
-		attributeValues.push_back(Format::Number(60. * outfit.FiringHeat() / outfit.Reload()));
-		attributesHeight += 20;
-	}
-	
-	if(outfit.FiringFuel() && outfit.Reload())
-	{
-		attributeLabels.push_back("firing fuel / second:");
-		attributeValues.push_back(Format::Number(60. * outfit.FiringFuel() / outfit.Reload()));
-		attributesHeight += 20;
-	}
-	
-	bool isContinuous = (outfit.Reload() <= 1);
-	attributeLabels.push_back("shots / second:");
+	bool isContinuous = (reload <= 1);
+	attributeLabels.emplace_back("shots / second:");
 	if(isContinuous)
-		attributeValues.push_back("continuous");
+		attributeValues.emplace_back("continuous");
 	else
-		attributeValues.push_back(Format::Number(60. / outfit.Reload()));
+		attributeValues.emplace_back(Format::Number(60. / reload));
 	attributesHeight += 20;
 	
 	double turretTurn = outfit.TurretTurn() * 60.;
 	if(turretTurn)
 	{
-		attributeLabels.push_back("turret turn rate:");
-		attributeValues.push_back(Format::Number(turretTurn));
+		attributeLabels.emplace_back("turret turn rate:");
+		attributeValues.emplace_back(Format::Number(turretTurn));
 		attributesHeight += 20;
 	}
 	int homing = outfit.Homing();
 	if(homing)
 	{
 		static const string skill[] = {
-			"no",
+			"none",
 			"poor",
 			"fair",
 			"good",
 			"excellent"
 		};
-		attributeLabels.push_back("homing:");
+		attributeLabels.emplace_back("homing:");
 		attributeValues.push_back(skill[max(0, min(4, homing))]);
 		attributesHeight += 20;
 	}
@@ -323,51 +331,49 @@ void OutfitInfoDisplay::UpdateAttributes(const Outfit &outfit)
 	for(unsigned i = 0; i < PERCENT_NAMES.size(); ++i)
 		if(percentValues[i])
 		{
-			int percent = 100. * percentValues[i] + .5;
+			int percent = lround(100. * percentValues[i]);
 			attributeLabels.push_back(PERCENT_NAMES[i]);
 			attributeValues.push_back(Format::Number(percent) + "%");
 			attributesHeight += 20;
 		}
 	
-	attributeLabels.push_back(string());
-	attributeValues.push_back(string());
+	// Pad the table.
+	attributeLabels.emplace_back();
+	attributeValues.emplace_back();
 	attributesHeight += 10;
 	
+	// Add per-shot values to the table. If the weapon fires continuously,
+	// the values have already been added.
+	if(!isContinuous)
+	{
+		static const string PER_SHOT = " / shot:";
+		for(unsigned i = 0; i < VALUE_NAMES.size(); ++i)
+			if(values[i])
+			{
+				attributeLabels.emplace_back(VALUE_NAMES[i] + PER_SHOT);
+				attributeValues.emplace_back(Format::Number(values[i]));
+				attributesHeight += 20;
+			}
+	}
+	
 	static const vector<string> OTHER_NAMES = {
-		"shield damage / shot:",
-		"hull damage / shot:",
-		"heat damage / shot:",
-		"ion damage / shot:",
-		"slowing damage / shot:",
-		"disruption damage / shot:",
-		"firing energy / shot:",
-		"firing heat / shot:",
-		"firing fuel / shot:",
 		"inaccuracy:",
 		"blast radius:",
 		"missile strength:",
-		"anti-missile:",
+		"anti-missile:"
 	};
-	vector<double> values = {
-		outfit.ShieldDamage(),
-		outfit.HullDamage(),
-		outfit.HeatDamage(),
-		outfit.IonDamage() * 100.,
-		outfit.SlowingDamage() * 100.,
-		outfit.DisruptionDamage() * 100.,
-		outfit.FiringEnergy(),
-		outfit.FiringHeat(),
-		outfit.FiringFuel(),
+	vector<double> otherValues = {
 		outfit.Inaccuracy(),
 		outfit.BlastRadius(),
 		static_cast<double>(outfit.MissileStrength()),
 		static_cast<double>(outfit.AntiMissile())
 	};
-	for(unsigned i = (isContinuous ? 9 : 0); i < OTHER_NAMES.size(); ++i)
-		if(values[i])
+	
+	for(unsigned i = 0; i < OTHER_NAMES.size(); ++i)
+		if(otherValues[i])
 		{
-			attributeLabels.push_back(OTHER_NAMES[i]);
-			attributeValues.push_back(Format::Number(values[i]));
+			attributeLabels.emplace_back(OTHER_NAMES[i]);
+			attributeValues.emplace_back(Format::Number(otherValues[i]));
 			attributesHeight += 20;
 		}
 }
