@@ -16,6 +16,7 @@ PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 
 #include "BankPanel.h"
 #include "Command.h"
+#include "ConversationPanel.h"
 #include "Dialog.h"
 #include "GameData.h"
 #include "FontSet.h"
@@ -93,32 +94,37 @@ void PlanetPanel::Draw()
 	if(player.IsDead())
 		return;
 	
-	const Ship *flagship = player.Flagship();
-	
 	Information info;
 	info.SetSprite("land", planet.Landscape());
-	bool hasAccess = planet.CanUseServices();
-	bool hasShip = false;
-	for(const auto &it : player.Ships())
-		if(it->GetSystem() == player.GetSystem() && !it->IsDisabled())
-		{
-			hasShip = true;
-			break;
-		}
+	
+	const Ship *flagship = player.Flagship();
 	if(flagship && flagship->CanBeFlagship())
 		info.SetCondition("has ship");
-	if(flagship && planet.IsInhabited() && planet.GetSystem()->HasTrade() && hasAccess)
-		info.SetCondition("has trade");
-	if(planet.IsInhabited() && hasAccess)
-		info.SetCondition("has bank");
-	if(flagship && planet.IsInhabited() && hasAccess)
-		info.SetCondition("is inhabited");
-	if(flagship && planet.HasSpaceport() && hasAccess)
-		info.SetCondition("has spaceport");
-	if(planet.HasShipyard() && hasAccess)
-		info.SetCondition("has shipyard");
-	if(hasShip && planet.HasOutfitter() && hasAccess)
-		info.SetCondition("has outfitter");
+	
+	if(planet.CanUseServices())
+	{
+		if(flagship && planet.IsInhabited())
+		{
+			info.SetCondition("is inhabited");
+			info.SetCondition("has bank");
+			if(system.HasTrade())
+				info.SetCondition("has trade");
+		}
+		
+		if(flagship && planet.HasSpaceport())
+			info.SetCondition("has spaceport");
+		
+		if(planet.HasShipyard())
+			info.SetCondition("has shipyard");
+		
+		if(planet.HasOutfitter())
+			for(const auto &it : player.Ships())
+				if(it->GetSystem() == &system && !it->IsDisabled())
+				{
+					info.SetCondition("has outfitter");
+					break;
+				}
+	}
 	
 	ui.Draw(info, this);
 	
@@ -129,7 +135,7 @@ void PlanetPanel::Draw()
 
 
 // Only override the ones you need; the default action is to return false.
-bool PlanetPanel::KeyDown(SDL_Keycode key, Uint16 mod, const Command &command)
+bool PlanetPanel::KeyDown(SDL_Keycode key, Uint16 mod, const Command &command, bool isNewPress)
 {
 	Panel *oldPanel = selectedPanel;
 	const Ship *flagship = player.Flagship();
@@ -141,45 +147,43 @@ bool PlanetPanel::KeyDown(SDL_Keycode key, Uint16 mod, const Command &command)
 	{
 		selectedPanel = nullptr;
 	}
-	else if(key == 't' && flagship && planet.IsInhabited() && planet.GetSystem()->HasTrade() && hasAccess)
+	else if(key == 't' && hasAccess && flagship && planet.IsInhabited() && system.HasTrade())
 	{
 		selectedPanel = trading.get();
 		GetUI()->Push(trading);
 	}
-	else if(key == 'b' && planet.IsInhabited() && hasAccess)
+	else if(key == 'b' && hasAccess && planet.IsInhabited())
 	{
 		selectedPanel = bank.get();
 		GetUI()->Push(bank);
 	}
-	else if(key == 'p' && flagship && planet.HasSpaceport() && hasAccess)
+	else if(key == 'p' && hasAccess && flagship && planet.HasSpaceport())
 	{
 		selectedPanel = spaceport.get();
+		if(isNewPress)
+			spaceport->UpdateNews();
 		GetUI()->Push(spaceport);
 	}
-	else if(key == 's' && planet.HasShipyard() && hasAccess)
+	else if(key == 's' && hasAccess && planet.HasShipyard())
 	{
 		GetUI()->Push(new ShipyardPanel(player));
 		return true;
 	}
-	else if(key == 'o' && planet.HasOutfitter() && hasAccess)
+	else if(key == 'o' && hasAccess && planet.HasOutfitter())
 	{
-		bool hasShip = false;
 		for(const auto &it : player.Ships())
-			if(it->GetSystem() == player.GetSystem() && !it->IsDisabled())
+			if(it->GetSystem() == &system && !it->IsDisabled())
 			{
-				hasShip = true;
-				break;
+				GetUI()->Push(new OutfitterPanel(player));
+				return true;
 			}
-		if(hasShip)
-			GetUI()->Push(new OutfitterPanel(player));
-		return true;
 	}
-	else if(key == 'j' && flagship && planet.IsInhabited() && hasAccess)
+	else if(key == 'j' && hasAccess && flagship && planet.IsInhabited())
 	{
 		GetUI()->Push(new MissionPanel(player));
 		return true;
 	}
-	else if(key == 'h' && flagship && planet.IsInhabited() && hasAccess)
+	else if(key == 'h' && hasAccess && flagship && planet.IsInhabited())
 	{
 		selectedPanel = hiring.get();
 		GetUI()->Push(hiring);
@@ -198,7 +202,7 @@ bool PlanetPanel::KeyDown(SDL_Keycode key, Uint16 mod, const Command &command)
 		return false;
 	
 	// If we are here, it is because something happened to change the selected
-	// panel. So, we need to pop the old selected panel:
+	// planet UI panel. So, we need to pop the old selected panel:
 	if(oldPanel)
 		GetUI()->Pop(oldPanel);
 	
@@ -214,6 +218,10 @@ void PlanetPanel::TakeOffIfReady()
 			&& !GetUI()->IsTop(spaceport.get()) && !GetUI()->IsTop(hiring.get()))
 		return;
 	
+	// If something happens here that cancels the order to take off, don't try
+	// to take off until the button is clicked again.
+	requestedLaunch = false;
+	
 	// Check for any landing missions that have not been offered.
 	Mission *mission = player.MissionToOffer(Mission::LANDING);
 	if(mission)
@@ -229,7 +237,23 @@ void PlanetPanel::TakeOffIfReady()
 		return;
 	}
 	
-	// The checks that follow are typically cause by parking or selling
+	// Check if any of the player's ships are configured in such a way that they
+	// will be impossible to fly.
+	for(const shared_ptr<Ship> &ship : player.Ships())
+	{
+		if(ship->GetSystem() != &system || ship->IsDisabled() || ship->IsParked())
+			continue;
+		
+		string check = ship->FlightCheck();
+		if(!check.empty() && check.back() == '!')
+		{
+			GetUI()->Push(new ConversationPanel(player,
+				*GameData::Conversations().Get("flight check: " + check), nullptr, ship));
+			return;
+		}
+	}
+	
+	// The checks that follow are typically caused by parking or selling
 	// ships or changing outfits.
 	const Ship *flagship = player.Flagship();
 	
@@ -237,14 +261,14 @@ void PlanetPanel::TakeOffIfReady()
 	// ship can't hold the required crew, count it as having no fireable
 	// crew rather than a negative number.
 	const CargoHold &cargo = player.Cargo();
-	int overbooked = -cargo.Bunks() - max(0, flagship->Crew() - flagship->RequiredCrew());
+	int overbooked = -cargo.BunksFree() - max(0, flagship->Crew() - flagship->RequiredCrew());
 	int missionCargoToSell = cargo.MissionCargoSize() - cargo.Size();
 	// Will you have to sell something other than regular cargo?
 	int cargoToSell = -(cargo.Free() + cargo.CommoditiesSize());
 	int droneCount = 0;
 	int fighterCount = 0;
 	for(const auto &it : player.Ships())
-		if(!it->IsParked() && !it->IsDisabled() && it->GetSystem() == player.GetSystem())
+		if(!it->IsParked() && !it->IsDisabled() && it->GetSystem() == &system)
 		{
 			const string &category = it->Attributes().Category();
 			droneCount += (category == "Drone") - it->BaysFree(false);
@@ -300,7 +324,6 @@ void PlanetPanel::TakeOffIfReady()
 		}
 		out << " Are you sure you want to continue?";
 		GetUI()->Push(new Dialog(this, &PlanetPanel::TakeOff, out.str()));
-		requestedLaunch = false;
 		return;
 	}
 	
