@@ -16,6 +16,7 @@ PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 #include "Color.h"
 #include "Dialog.h"
 #include "Files.h"
+#include "Font.h"
 #include "FontSet.h"
 #include "GameData.h"
 #include "Information.h"
@@ -39,14 +40,18 @@ using namespace std;
 
 namespace {
 	// Settings that require special handling.
-	static const string ZOOM_FACTOR = "Main zoom factor";
-	static const string VIEW_ZOOM_FACTOR = "View zoom factor";
-	static const string EXPEND_AMMO = "Escorts expend ammo";
-	static const string TURRET_TRACKING = "Turret tracking";
-	static const string FOCUS_PREFERENCE = "Turrets focus fire";
-	static const string FRUGAL_ESCORTS = "Escorts use ammo frugally";
-	static const string REACTIVATE_HELP = "Reactivate first-time help";
-	static const string SCROLL_SPEED = "Scroll speed";
+	const string ZOOM_FACTOR = "Main zoom factor";
+	const int ZOOM_FACTOR_MIN = 100;
+	const int ZOOM_FACTOR_MAX = 200;
+	const int ZOOM_FACTOR_INCREMENT = 10;
+	const string VIEW_ZOOM_FACTOR = "View zoom factor";
+	const string EXPEND_AMMO = "Escorts expend ammo";
+	const string TURRET_TRACKING = "Turret tracking";
+	const string FOCUS_PREFERENCE = "Turrets focus fire";
+	const string FRUGAL_ESCORTS = "Escorts use ammo frugally";
+	const string REACTIVATE_HELP = "Reactivate first-time help";
+	const string SCROLL_SPEED = "Scroll speed";
+	const string FIGHTER_REPAIR = "Repair fighters in";
 }
 
 
@@ -88,7 +93,7 @@ void PreferencesPanel::Draw()
 
 
 
-bool PreferencesPanel::KeyDown(SDL_Keycode key, Uint16 mod, const Command &command)
+bool PreferencesPanel::KeyDown(SDL_Keycode key, Uint16 mod, const Command &command, bool isNewPress)
 {
 	if(static_cast<unsigned>(editing) < zones.size())
 	{
@@ -136,33 +141,32 @@ bool PreferencesPanel::Click(int x, int y, int clicks)
 		{
 			if(zone.Value() == ZOOM_FACTOR)
 			{
-				int currentZoom = Screen::Zoom();
-				int newZoom = currentZoom == 100 ? 150 : currentZoom == 150 ? 200 : 100;
+				int newZoom = Screen::UserZoom() + ZOOM_FACTOR_INCREMENT;
 				Screen::SetZoom(newZoom);
-				// Make sure there is enough vertical space for the full UI.
-				if(Screen::Height() < 700)
+				if(newZoom > ZOOM_FACTOR_MAX || Screen::Zoom() != newZoom)
 				{
 					// Notify the user why setting the zoom any higher isn't permitted.
-					// Only show this if it's not possible to zoom the view at all,
-					// because otherwise if the user is toggling between 100 and 150 the
-					// dialog will show every time, which is annoying.
-					if(newZoom == 150)
+					// Only show this if it's not possible to zoom the view at all, as
+					// otherwise the dialog will show every time, which is annoying.
+					if(newZoom == ZOOM_FACTOR_MIN + ZOOM_FACTOR_INCREMENT)
 						GetUI()->Push(new Dialog(
 							"Your screen resolution is too low to support a zoom level above 100%."));
-					Screen::SetZoom(100);
+					Screen::SetZoom(ZOOM_FACTOR_MIN);
 				}
 				// Convert to raw window coordinates, at the new zoom level.
 				point *= Screen::Zoom() / 100.;
 				point += .5 * Point(Screen::RawWidth(), Screen::RawHeight());
 				SDL_WarpMouseInWindow(nullptr, point.X(), point.Y());
 			}
-			if(zone.Value() == VIEW_ZOOM_FACTOR)
+			else if(zone.Value() == VIEW_ZOOM_FACTOR)
 			{
 				// Increase the zoom factor unless it is at the maximum. In that
 				// case, cycle around to the lowest zoom factor.
 				if(!Preferences::ZoomViewIn())
 					while(Preferences::ZoomViewOut()) {}
 			}
+			
+			// Update saved preferences.
 			if(zone.Value() == EXPEND_AMMO)
 				Preferences::ToggleAmmoUsage();
 			else if(zone.Value() == TURRET_TRACKING)
@@ -187,7 +191,10 @@ bool PreferencesPanel::Click(int x, int y, int clicks)
 	
 	for(const auto &zone : pluginZones)
 		if(zone.Contains(point))
+		{
 			selectedPlugin = zone.Value();
+			break;
+		}
 	
 	return true;
 }
@@ -218,6 +225,7 @@ bool PreferencesPanel::Hover(int x, int y)
 
 
 
+// Change the value being hovered over in the direction of the scroll.
 bool PreferencesPanel::Scroll(double dx, double dy)
 {
 	if(!dy || hoverPreference.empty())
@@ -225,19 +233,15 @@ bool PreferencesPanel::Scroll(double dx, double dy)
 	
 	if(hoverPreference == ZOOM_FACTOR)
 	{
-		int zoom = Screen::Zoom();
-		if(dy < 0. && zoom > 100)
-			zoom -= 50;
-		if(dy > 0. && zoom < 200)
-			zoom += 50;
+		int zoom = Screen::UserZoom();
+		if(dy < 0. && zoom > ZOOM_FACTOR_MIN)
+			zoom -= ZOOM_FACTOR_INCREMENT;
+		if(dy > 0. && zoom < ZOOM_FACTOR_MAX)
+			zoom += ZOOM_FACTOR_INCREMENT;
 		
 		Screen::SetZoom(zoom);
-		// Make sure there is enough vertical space for the full UI.
-		while(Screen::Height() < 700 && zoom > 100)
-		{
-			zoom -= 50;
-			Screen::SetZoom(zoom);
-		}
+		if (Screen::Zoom() != zoom)
+			Screen::SetZoom(Screen::Zoom());
 		
 		// Convert to raw window coordinates, at the new zoom level.
 		Point point = hoverPoint * (Screen::Zoom() / 100.);
@@ -274,13 +278,13 @@ void PreferencesPanel::EndEditing()
 
 void PreferencesPanel::DrawControls()
 {
-	Color back = *GameData::Colors().Get("faint");
-	Color dim = *GameData::Colors().Get("dim");
-	Color medium = *GameData::Colors().Get("medium");
-	Color bright = *GameData::Colors().Get("bright");
+	const Color &back = *GameData::Colors().Get("faint");
+	const Color &dim = *GameData::Colors().Get("dim");
+	const Color &medium = *GameData::Colors().Get("medium");
+	const Color &bright = *GameData::Colors().Get("bright");
 	
 	// Check for conflicts.
-	Color red(.3, 0., 0., .3);
+	Color red(.3f, 0.f, 0.f, .3f);
 	
 	Table table;
 	table.AddColumn(-115, Table::LEFT);
@@ -398,10 +402,10 @@ void PreferencesPanel::DrawControls()
 
 void PreferencesPanel::DrawSettings()
 {
-	Color back = *GameData::Colors().Get("faint");
-	Color dim = *GameData::Colors().Get("dim");
-	Color medium = *GameData::Colors().Get("medium");
-	Color bright = *GameData::Colors().Get("bright");
+	const Color &back = *GameData::Colors().Get("faint");
+	const Color &dim = *GameData::Colors().Get("dim");
+	const Color &medium = *GameData::Colors().Get("medium");
+	const Color &bright = *GameData::Colors().Get("bright");
 	
 	Table table;
 	table.AddColumn(-115, Table::LEFT);
@@ -425,20 +429,24 @@ void PreferencesPanel::DrawSettings()
 		"Automatic aiming",
 		"Automatic firing",
 		EXPEND_AMMO,
+		FIGHTER_REPAIR,
 		TURRET_TRACKING,
-		"",
+		"\n",
 		"Performance",
 		"Show CPU / GPU load",
 		"Render motion blur",
 		"Reduce large graphics",
 		"Draw background haze",
 		"Show hyperspace flash",
-		"\n",
+		"",
 		"Other",
+		"Clickable radar display",
+		"Hide unexplored map regions",
 		REACTIVATE_HELP,
+		"Rehire extra crew when lost",
 		SCROLL_SPEED,
-		"Warning siren",
-		"Hide unexplored map regions"
+		"Show escort systems on map",
+		"Warning siren"
 	};
 	bool isCategory = true;
 	for(const string &setting : SETTINGS)
@@ -466,13 +474,14 @@ void PreferencesPanel::DrawSettings()
 		// Record where this setting is displayed, so the user can click on it.
 		prefZones.emplace_back(table.GetCenterPoint(), table.GetRowSize(), setting);
 		
-		// Get the "on / off" text for this setting.
+		// Get the "on / off" text for this setting. Setting "isOn"
+		// draws the setting "bright" (i.e. the setting is active).
 		bool isOn = Preferences::Has(setting);
 		string text;
 		if(setting == ZOOM_FACTOR)
 		{
-			isOn = true;
-			text = to_string(Screen::Zoom());
+			isOn = Screen::UserZoom() == Screen::Zoom();
+			text = to_string(Screen::UserZoom());
 		}
 		else if(setting == VIEW_ZOOM_FACTOR)
 		{
@@ -486,17 +495,36 @@ void PreferencesPanel::DrawSettings()
 			isOn = true;
 			text = Preferences::Has(FOCUS_PREFERENCE) ? "focused" : "opportunistic";
 		}
+		else if(setting == FIGHTER_REPAIR)
+		{
+			isOn = true;
+			text = Preferences::Has(FIGHTER_REPAIR) ? "parallel" : "series";
+		}
 		else if(setting == REACTIVATE_HELP)
 		{
 			// Check how many help messages have been displayed.
 			const map<string, string> &help = GameData::HelpTemplates();
 			int shown = 0;
+			int total = 0;
 			for(const auto &it : help)
-				shown += Preferences::Has("help: " + it.first);
+			{
+				// Don't count certain special help messages that are always
+				// active for new players.
+				bool special = false;
+				const string SPECIAL_HELP[] = {"basics", "lost"};
+				for(const string &str : SPECIAL_HELP)
+					if(it.first.find(str) == 0)
+						special = true;
+				
+				if(!special)
+				{
+					++total;
+					shown += Preferences::Has("help: " + it.first);
+				}
+			}
 			
-			// Don't count the "basic help" messages in the total.
 			if(shown)
-				text = to_string(shown) + " / " + to_string(help.size() - 2);
+				text = to_string(shown) + " / " + to_string(total);
 			else
 			{
 				isOn = true;
@@ -522,9 +550,9 @@ void PreferencesPanel::DrawSettings()
 
 void PreferencesPanel::DrawPlugins()
 {
-	Color back = *GameData::Colors().Get("faint");
-	Color medium = *GameData::Colors().Get("medium");
-	Color bright = *GameData::Colors().Get("bright");
+	const Color &back = *GameData::Colors().Get("faint");
+	const Color &medium = *GameData::Colors().Get("medium");
+	const Color &bright = *GameData::Colors().Get("bright");
 	
 	Table table;
 	table.AddColumn(-115, Table::LEFT);
@@ -536,18 +564,20 @@ void PreferencesPanel::DrawPlugins()
 	table.Draw("Installed plugins:", bright);
 	table.DrawGap(5);
 	
-	for(const auto &it : GameData::PluginAboutText())
+	const int MAX_TEXT_WIDTH = 230;
+	const Font &font = FontSet::Get(14);
+	for(const pair<string, string> &plugin : GameData::PluginAboutText())
 	{
-		pluginZones.emplace_back(table.GetCenterPoint(), table.GetRowSize(), it.first);
+		pluginZones.emplace_back(table.GetCenterPoint(), table.GetRowSize(), plugin.first);
 		
-		bool isSelected = (it.first == selectedPlugin);
-		if(isSelected || it.first == hoverPlugin)
+		bool isSelected = (plugin.first == selectedPlugin);
+		if(isSelected || plugin.first == hoverPlugin)
 			table.DrawHighlight(back);
-		table.Draw(it.first, isSelected ? bright : medium);
+		table.Draw(font.TruncateMiddle(plugin.first, MAX_TEXT_WIDTH), isSelected ? bright : medium);
 		
 		if(isSelected)
 		{
-			const Sprite *sprite = SpriteSet::Get(it.first);
+			const Sprite *sprite = SpriteSet::Get(plugin.first);
 			Point top(15., firstY);
 			if(sprite)
 			{
@@ -556,10 +586,10 @@ void PreferencesPanel::DrawPlugins()
 				top.Y() += sprite->Height() + 10.;
 			}
 			
-			WrappedText wrap(FontSet::Get(14));
-			wrap.SetWrapWidth(230);
-			static const string empty = "(No description given.)";
-			wrap.Wrap(it.second.empty() ? empty : it.second);
+			WrappedText wrap(font);
+			wrap.SetWrapWidth(MAX_TEXT_WIDTH);
+			static const string EMPTY = "(No description given.)";
+			wrap.Wrap(plugin.second.empty() ? EMPTY : plugin.second);
 			wrap.Draw(top, medium);
 		}
 	}
