@@ -51,7 +51,7 @@ void SavedGame::Load(const string &path)
 			for(const DataNode &child : node)
 				if(child.Token(0) == "credits" && child.Size() >= 2)
 				{
-					credits = Format::Number(child.Value(1));
+					credits = Format::Credits(child.Value(1));
 					break;
 				}
 		}
