@@ -56,13 +56,21 @@ void AdjustViewport(SDL_Window *window);
 int DoError(string message, SDL_Window *window = nullptr, SDL_GLContext context = nullptr);
 void Cleanup(SDL_Window *window, SDL_GLContext context);
 Conversation LoadConversation();
+#ifdef _WIN32
+void InitConsole();
+#endif
 
 
 
 int main(int argc, char *argv[])
 {
+#ifdef _WIN32
+	if(argc > 1)
+		InitConsole();
+#endif
 	Conversation conversation;
 	bool debugMode = false;
+	bool loadOnly = false;
 	for(const char *const *it = argv + 1; *it; ++it)
 	{
 		string arg = *it;
@@ -80,14 +88,26 @@ int main(int argc, char *argv[])
 			conversation = LoadConversation();
 		else if(arg == "-d" || arg == "--debug")
 			debugMode = true;
+		else if(arg == "-p" || arg == "--parse-save")
+			loadOnly = true;
 	}
 	PlayerInfo player;
 	
 	try {
+		// Begin loading the game data. Exit early if we are not using the UI.
+		if(!GameData::BeginLoad(argv))
+			return 0;
+		
+		// Load player data, including reference-checking.
+		player.LoadRecent();
+		if(loadOnly)
+		{
+			cout << "Parse completed." << endl;
+			return 0;
+		}
+		
 		SDL_Init(SDL_INIT_VIDEO);
 		
-		// Begin loading the game data.
-		GameData::BeginLoad(argv);
 		Audio::Init(GameData::Sources());
 		
 		// On Windows, make sure that the sleep timer has at least 1 ms resolution
@@ -96,8 +116,6 @@ int main(int argc, char *argv[])
 		timeBeginPeriod(1);
 #endif
 		
-		player.LoadRecent();
-		
 		// Check how big the window can be.
 		SDL_DisplayMode mode;
 		if(SDL_GetCurrentDisplayMode(0, &mode))
@@ -231,8 +249,12 @@ int main(int argc, char *argv[])
 		bool isPaused = false;
 		// If fast forwarding, keep track of whether the current frame should be drawn.
 		int skipFrame = 0;
+		// Limit how quickly fullscreen mode can be toggled.
+		int toggleTimeout = 0;
 		while(!menuPanels.IsDone())
 		{
+			if(toggleTimeout)
+				--toggleTimeout;
 			// Handle any events that occurred in this frame.
 			SDL_Event event;
 			while(SDL_PollEvent(&event))
@@ -268,13 +290,14 @@ int main(int argc, char *argv[])
 					if(!isFullscreen)
 						SDL_GetWindowSize(window, &windowWidth, &windowHeight);
 				}
-				else if(event.type == SDL_KEYDOWN
+				else if(event.type == SDL_KEYDOWN && !toggleTimeout
 						&& (Command(event.key.keysym.sym).Has(Command::FULLSCREEN)
 						|| (event.key.keysym.sym == SDLK_RETURN && (event.key.keysym.mod & KMOD_ALT))))
 				{
 					// Toggle full-screen mode. This will generate a window size
 					// change event, so no need to adjust the viewport here.
 					isFullscreen = !isFullscreen;
+					toggleTimeout = 30;
 					if(!isFullscreen)
 					{
 						SDL_SetWindowFullscreen(window, 0);
@@ -293,9 +316,9 @@ int main(int argc, char *argv[])
 			
 			// In fullscreen mode, hide the cursor if inactive for ten seconds,
 			// but only if the player is flying around in the main view.
+			bool inFlight = (menuPanels.IsEmpty() && gamePanels.Root() == gamePanels.Top());
 			++cursorTime;
-			bool shouldShowCursor = (!isFullscreen || cursorTime < 600 
-				|| !menuPanels.IsEmpty() || gamePanels.Root() != gamePanels.Top());
+			bool shouldShowCursor = (!isFullscreen || cursorTime < 600 || !inFlight);
 			if(shouldShowCursor != showCursor)
 			{
 				showCursor = shouldShowCursor;
@@ -308,7 +331,7 @@ int main(int argc, char *argv[])
 			// Caps lock slows the frame rate in debug mode, but raises it in
 			// normal mode. Slowing eases in and out over a couple of frames.
 			bool fastForward = false;
-			if(mod & KMOD_CAPS)
+			if((mod & KMOD_CAPS) && inFlight)
 			{
 				if(debugMode)
 				{
@@ -343,8 +366,8 @@ int main(int argc, char *argv[])
 			timer.Wait();
 		}
 		
-		// If you quit while landed on a planet, save the game.
-		if(player.GetPlanet())
+		// If you quit while landed on a planet, save the game - if you did anything.
+		if(player.GetPlanet() && gamePanels.CanSave())
 			player.Save();
 		
 		// Remember the window state.
@@ -374,14 +397,15 @@ void PrintHelp()
 	cerr << "Command line options:" << endl;
 	cerr << "    -h, --help: print this help message." << endl;
 	cerr << "    -v, --version: print version information." << endl;
-	cerr << "    -s, --ships: print table of ship statistics." << endl;
-	cerr << "    -w, --weapons: print table of weapon statistics." << endl;
+	cerr << "    -s, --ships: print table of ship statistics, then exit." << endl;
+	cerr << "    -w, --weapons: print table of weapon statistics, then exit." << endl;
 	cerr << "    -t, --talk: read and display a conversation from STDIN." << endl;
 	cerr << "    -r, --resources <path>: load resources from given directory." << endl;
 	cerr << "    -c, --config <path>: save user's files to given directory." << endl;
-	cerr << "    -d, --debug: turn on debugging features (e.g. caps lock slow motion)." << endl;
+	cerr << "    -d, --debug: turn on debugging features (e.g. Caps Lock slows down instead of speeds up)." << endl;
+	cerr << "    -p, --parse-save: load the most recent saved game and inspect it for content errors" << endl;
 	cerr << endl;
-	cerr << "Report bugs to: mzahniser@gmail.com" << endl;
+	cerr << "Report bugs to: <https://github.com/endless-sky/endless-sky/issues>" << endl;
 	cerr << "Home page: <https://endless-sky.github.io>" << endl;
 	cerr << endl;
 }
@@ -391,7 +415,7 @@ void PrintHelp()
 void PrintVersion()
 {
 	cerr << endl;
-	cerr << "Endless Sky 0.9.8" << endl;
+	cerr << "Endless Sky 0.9.9" << endl;
 	cerr << "License GPLv3+: GNU GPL version 3 or later: <https://gnu.org/licenses/gpl.html>" << endl;
 	cerr << "This is free software: you are free to change and redistribute it." << endl;
 	cerr << "There is NO WARRANTY, to the extent permitted by law." << endl;
@@ -403,25 +427,20 @@ void PrintVersion()
 void SetIcon(SDL_Window *window)
 {
 	// Load the icon file.
-	ImageBuffer *buffer = ImageBuffer::Read(Files::Resources() + "icon.png");
-	if(!buffer)
+	ImageBuffer buffer;
+	if(!buffer.Read(Files::Resources() + "icon.png"))
 		return;
-	if(!buffer->Pixels() || !buffer->Width() || !buffer->Height())
-	{
-		delete buffer;
+	if(!buffer.Pixels() || !buffer.Width() || !buffer.Height())
 		return;
-	}
 	
 	// Convert the icon to an SDL surface.
-	SDL_Surface *surface = SDL_CreateRGBSurfaceFrom(buffer->Pixels(), buffer->Width(), buffer->Height(),
-		32, 4 * buffer->Width(), 0x00FF0000, 0x0000FF00, 0x000000FF, 0xFF000000);
+	SDL_Surface *surface = SDL_CreateRGBSurfaceFrom(buffer.Pixels(), buffer.Width(), buffer.Height(),
+		32, 4 * buffer.Width(), 0x00FF0000, 0x0000FF00, 0x000000FF, 0xFF000000);
 	if(surface)
 	{
 		SDL_SetWindowIcon(window, surface);
 		SDL_FreeSurface(surface);
 	}
-	// Free the image buffer.
-	delete buffer;
 }
 
 
@@ -449,10 +468,6 @@ void AdjustViewport(SDL_Window *window)
 	drawWidth = (drawWidth * roundWidth) / width;
 	drawHeight = (drawHeight * roundHeight) / height;
 	glViewport(0, 0, drawWidth, drawHeight);
-	
-	// Make sure the zoom factor is not set too high for the full UI to fit.
-	if(Screen::Height() < 700)
-		Screen::SetZoom(100);
 }
 
 
@@ -470,8 +485,8 @@ int DoError(string message, SDL_Window *window, SDL_GLContext context)
 		message += "\")";
 	}
 	
-	// Print the error message in the terminal.
-	cerr << message << endl;
+	// Print the error message in the terminal and the error file.
+	Files::LogError(message);
 	
 	// Show the error message both in a message box and in the terminal.
 	SDL_MessageBoxData box;
@@ -546,3 +561,28 @@ Conversation LoadConversation()
 	return conversation.Substitute(subs);
 }
 
+
+
+#ifdef _WIN32
+void InitConsole()
+{
+	// If both stdout and stderr are already initialized (e.g. writing to a file), do nothing.
+	const int UNINITIALIZED = -2;
+	bool redirectStdout = _fileno(stdout) == UNINITIALIZED;
+	bool redirectStderr = _fileno(stderr) == UNINITIALIZED;
+	bool redirectStdin = _fileno(stdin) == UNINITIALIZED;
+
+	if(!redirectStdout && !redirectStderr && !redirectStdin)
+		return;
+	
+	if(!AttachConsole(ATTACH_PARENT_PROCESS) && !AllocConsole())
+		return;
+
+	if(redirectStdout && freopen("CONOUT$", "w", stdout))
+		setvbuf(stdout, nullptr, _IOFBF, 4096);
+	if(redirectStderr && freopen("CONOUT$", "w", stderr))
+		setvbuf(stderr, nullptr, _IOLBF, 1024);
+	if(redirectStdin && freopen("CONIN$", "r", stdin))
+		setvbuf(stdin, nullptr, _IONBF, 0);
+}
+#endif
